# This file has been autogenerated by the pywayland scanner

# Copyright © 2016, 2023 Red Hat
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from __future__ import annotations

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource

from .wayland import WlSurface
from .wayland import WlSurfaceProxy


class XdgSystemBellV1(Interface):
    """System bell

    This global interface enables clients to ring the system bell.

    Warning! The protocol described in this file is currently in the testing
    phase. Backward compatible changes may be added together with the
    corresponding interface version bump. Backward incompatible changes can
    only be done by creating a new major version of the extension.
    """

    name = "xdg_system_bell_v1"
    version = 1


class XdgSystemBellV1Resource(Resource[XdgSystemBellV1]):
    interface = XdgSystemBellV1


class XdgSystemBellV1Proxy(Proxy[XdgSystemBellV1]):
    interface = XdgSystemBellV1

    @XdgSystemBellV1.request()
    def destroy(self) -> None:
        """Destroy the system bell object

        Notify that the object will no longer be used.
        """
        self._marshal(0)
        self._destroy()

    @XdgSystemBellV1.request(
        Argument(ArgumentType.Object, interface=WlSurface, nullable=True),
    )
    def ring(self, surface: WlSurfaceProxy | None) -> None:
        """Ring the system bell

        This requests rings the system bell on behalf of a client. How ringing
        the bell is implemented is up to the compositor. It may be an audible
        sound, a visual feedback of some kind, or any other thing including
        nothing.

        The passed surface should correspond to a toplevel like surface role,
        or be null, meaning the client doesn't have a particular toplevel it
        wants to associate the bell ringing with. See the xdg-shell protocol
        extension for a toplevel like surface role.

        :param surface:
            associated surface
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface` or `None`
        """
        self._marshal(1, surface)


class XdgSystemBellV1Global(Global[XdgSystemBellV1]):
    interface = XdgSystemBellV1


XdgSystemBellV1._gen_c()
XdgSystemBellV1.proxy_class = XdgSystemBellV1Proxy
XdgSystemBellV1.resource_class = XdgSystemBellV1Resource
XdgSystemBellV1.global_class = XdgSystemBellV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2024 Neal Gompa
# Copyright © 2024 Xaver Hugl
# Copyright © 2024 Matthias Klumpp
# Copyright © 2024 Vlad Zahorodnii
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from __future__ import annotations

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource

from .wayland import WlPointer
from .wayland import WlPointerProxy
from .wayland import WlSurface
from .wayland import WlSurfaceProxy


class WpPointerWarpV1(Interface):
    """Reposition the pointer to a location on a surface

    This global interface allows applications to request the pointer to be
    moved to a position relative to a
    :class:`~pywayland.protocol.wayland.WlSurface`.

    Note that if the desired behavior is to constrain the pointer to an area or
    lock it to a position, this protocol does not provide a reliable way to do
    that. The pointer constraint and pointer lock protocols should be used for
    those use cases instead.

    Warning! The protocol described in this file is currently in the testing
    phase. Backward compatible changes may be added together with the
    corresponding interface version bump. Backward incompatible changes can
    only be done by creating a new major version of the extension.
    """

    name = "wp_pointer_warp_v1"
    version = 1


class WpPointerWarpV1Resource(Resource[WpPointerWarpV1]):
    interface = WpPointerWarpV1


class WpPointerWarpV1Proxy(Proxy[WpPointerWarpV1]):
    interface = WpPointerWarpV1

    @WpPointerWarpV1.request()
    def destroy(self) -> None:
        """Destroy the warp manager

        Destroy the pointer warp manager.
        """
        self._marshal(0)
        self._destroy()

    @WpPointerWarpV1.request(
        Argument(ArgumentType.Object, interface=WlSurface),
        Argument(ArgumentType.Object, interface=WlPointer),
        Argument(ArgumentType.Fixed),
        Argument(ArgumentType.Fixed),
        Argument(ArgumentType.Uint),
    )
    def warp_pointer(self, surface: WlSurfaceProxy, pointer: WlPointerProxy, x: float, y: float, serial: int) -> None:
        """Reposition the pointer

        Request the compositor to move the pointer to a surface-local position.
        Whether or not the compositor honors the request is implementation
        defined, but it should - honor it if the surface has pointer focus,
        including   when it has an implicit pointer grab - reject it if the
        enter serial is incorrect - reject it if the requested position is
        outside of the surface

        Note that the enter serial is valid for any surface of the client, and
        does not have to be from the surface the pointer is warped to.

        :param surface:
            surface to position the pointer on
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        :param pointer:
            the pointer that should be repositioned
        :type pointer:
            :class:`~pywayland.protocol.wayland.WlPointer`
        :param x:
        :type x:
            `ArgumentType.Fixed`
        :param y:
        :type y:
            `ArgumentType.Fixed`
        :param serial:
            serial number of the enter event
        :type serial:
            `ArgumentType.Uint`
        """
        self._marshal(1, surface, pointer, x, y, serial)


class WpPointerWarpV1Global(Global[WpPointerWarpV1]):
    interface = WpPointerWarpV1


WpPointerWarpV1._gen_c()
WpPointerWarpV1.proxy_class = WpPointerWarpV1Proxy
WpPointerWarpV1.resource_class = WpPointerWarpV1Resource
WpPointerWarpV1.global_class = WpPointerWarpV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright 2014 © Stephen "Lyude" Chandler Paul
# Copyright 2015-2024 © Red Hat, Inc.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from __future__ import annotations

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource

from .wayland import WlSeat
from .wayland import WlSeatProxy
from .wayland import WlSurface
from .wayland import WlSurfaceProxy


class ZwpTabletManagerV2(Interface):
    """Controller object for graphic tablet devices

    An object that provides access to the graphics tablets available on this
    system. All tablets are associated with a seat, to get access to the actual
    tablets, use :func:`ZwpTabletManagerV2.get_tablet_seat()`.
    """

    name = "zwp_tablet_manager_v2"
    version = 2


class ZwpTabletPadDialV2(Interface):
    """Pad dial

    A rotary control, e.g. a dial or a wheel.

    Events on a dial are logically grouped by the
    :func:`ZwpTabletPadDialV2.frame()` event.
    """

    name = "zwp_tablet_pad_dial_v2"
    version = 2


class ZwpTabletPadGroupV2(Interface):
    """A set of buttons, rings and strips

    A pad group describes a distinct (sub)set of buttons, rings and strips
    present in the tablet. The criteria of this grouping is usually positional,
    eg. if a tablet has buttons on the left and right side, 2 groups will be
    presented. The physical arrangement of groups is undisclosed and may change
    on the fly.

    Pad groups will announce their features during pad initialization. Between
    the corresponding :func:`ZwpTabletPadV2.group()
    <pywayland.protocol.tablet_v2.ZwpTabletPadV2.group>` event and
    :func:`ZwpTabletPadGroupV2.done()`, the pad group will announce the
    buttons, rings and strips contained in it, plus the number of supported
    modes.

    Modes are a mechanism to allow multiple groups of actions for every element
    in the pad group. The number of groups and available modes in each is
    persistent across device plugs. The current mode is user-switchable, it
    will be announced through the :func:`ZwpTabletPadGroupV2.mode_switch()`
    event both whenever it is switched, and after :func:`ZwpTabletPadV2.enter()
    <pywayland.protocol.tablet_v2.ZwpTabletPadV2.enter>`.

    The current mode logically applies to all elements in the pad group,
    although it is at clients' discretion whether to actually perform different
    actions, and/or issue the respective .set_feedback requests to notify the
    compositor. See the :func:`ZwpTabletPadGroupV2.mode_switch()` event for
    more details.
    """

    name = "zwp_tablet_pad_group_v2"
    version = 2


class ZwpTabletPadRingV2(Interface):
    """Pad ring

    A circular interaction area, such as the touch ring on the Wacom Intuos Pro
    series tablets.

    Events on a ring are logically grouped by the
    :func:`ZwpTabletPadRingV2.frame()` event.
    """

    name = "zwp_tablet_pad_ring_v2"
    version = 2

    class source(enum.IntEnum):
        finger = 1


class ZwpTabletPadStripV2(Interface):
    """Pad strip

    A linear interaction area, such as the strips found in Wacom Cintiq models.

    Events on a strip are logically grouped by the
    :func:`ZwpTabletPadStripV2.frame()` event.
    """

    name = "zwp_tablet_pad_strip_v2"
    version = 2

    class source(enum.IntEnum):
        finger = 1


class ZwpTabletPadV2(Interface):
    """A set of buttons, rings, strips and dials

    A pad device is a set of buttons, rings, strips and dials usually
    physically present on the tablet device itself. Some exceptions exist where
    the pad device is physically detached, e.g. the Wacom ExpressKey Remote.

    Pad devices have no axes that control the cursor and are generally
    auxiliary devices to the tool devices used on the tablet surface.

    A pad device has a number of static characteristics, e.g. the number of
    rings. These capabilities are sent in an event sequence after the
    :func:`ZwpTabletSeatV2.pad_added()
    <pywayland.protocol.tablet_v2.ZwpTabletSeatV2.pad_added>` event before any
    actual events from this pad. This initial event sequence is terminated by a
    :func:`ZwpTabletPadV2.done()` event.

    All pad features (buttons, rings, strips and dials) are logically divided
    into groups and all pads have at least one group. The available groups are
    notified through the :func:`ZwpTabletPadV2.group()` event; the compositor
    will emit one event per group before emitting
    :func:`ZwpTabletPadV2.done()`.

    Groups may have multiple modes. Modes allow clients to map multiple actions
    to a single pad feature. Only one mode can be active per group, although
    different groups may have different active modes.
    """

    name = "zwp_tablet_pad_v2"
    version = 2

    class button_state(enum.IntEnum):
        released = 0
        pressed = 1


class ZwpTabletSeatV2(Interface):
    """Controller object for graphic tablet devices of a seat

    An object that provides access to the graphics tablets available on this
    seat. After binding to this interface, the compositor sends a set of
    :func:`ZwpTabletSeatV2.tablet_added()` and
    :func:`ZwpTabletSeatV2.tool_added()` events.
    """

    name = "zwp_tablet_seat_v2"
    version = 2


class ZwpTabletToolV2(Interface):
    """A physical tablet tool

    An object that represents a physical tool that has been, or is currently in
    use with a tablet in this seat. Each :class:`ZwpTabletToolV2` object stays
    valid until the client destroys it; the compositor reuses the
    :class:`ZwpTabletToolV2` object to indicate that the object's respective
    physical tool has come into proximity of a tablet again.

    A :class:`ZwpTabletToolV2` object's relation to a physical tool depends on
    the tablet's ability to report serial numbers. If the tablet supports this
    capability, then the object represents a specific physical tool and can be
    identified even when used on multiple tablets.

    A tablet tool has a number of static characteristics, e.g. tool type,
    hardware_serial and capabilities. These capabilities are sent in an event
    sequence after the :func:`ZwpTabletSeatV2.tool_added()
    <pywayland.protocol.tablet_v2.ZwpTabletSeatV2.tool_added>` event before any
    actual events from this tool. This initial event sequence is terminated by
    a :func:`ZwpTabletToolV2.done()` event.

    Tablet tool events are grouped by :func:`ZwpTabletToolV2.frame()` events.
    Any events received before a :func:`ZwpTabletToolV2.frame()` event should
    be considered part of the same hardware state change.
    """

    name = "zwp_tablet_tool_v2"
    version = 2

    class type(enum.IntEnum):
        pen = 0x140
        eraser = 0x141
        brush = 0x142
        pencil = 0x143
        airbrush = 0x144
        finger = 0x145
        mouse = 0x146
        lens = 0x147

    class capability(enum.IntEnum):
        tilt = 1
        pressure = 2
        distance = 3
        rotation = 4
        slider = 5
        wheel = 6

    class button_state(enum.IntEnum):
        released = 0
        pressed = 1

    class error(enum.IntEnum):
        role = 0


class ZwpTabletV2(Interface):
    """Graphics tablet device

    The :class:`ZwpTabletV2` interface represents one graphics tablet device.
    The tablet interface itself does not generate events; all events are
    generated by :class:`~pywayland.protocol.tablet_v2.ZwpTabletToolV2` objects
    when in proximity above a tablet.

    A tablet has a number of static characteristics, e.g. device name and
    pid/vid. These capabilities are sent in an event sequence after the
    :func:`ZwpTabletSeatV2.tablet_added()
    <pywayland.protocol.tablet_v2.ZwpTabletSeatV2.tablet_added>` event. This
    initial event sequence is terminated by a :func:`ZwpTabletV2.done()` event.
    """

    name = "zwp_tablet_v2"
    version = 2

    class bustype(enum.IntEnum):
        usb = 3
        bluetooth = 5
        virtual = 6
        serial = 17
        i2c = 24


class ZwpTabletManagerV2Resource(Resource[ZwpTabletManagerV2]):
    interface = ZwpTabletManagerV2


class ZwpTabletPadDialV2Resource(Resource[ZwpTabletPadDialV2]):
    interface = ZwpTabletPadDialV2

    @ZwpTabletPadDialV2.event(
        Argument(ArgumentType.Int),
    )
    def delta(self, value120: int) -> None:
        """Delta movement

        Sent whenever the position on a dial changes.

        This event carries the wheel delta as multiples or fractions of 120
        with each multiple of 120 representing one logical wheel detent. For
        example, an axis_value120 of 30 is one quarter of a logical wheel step
        in the positive direction, a value120 of -240 are two logical wheel
        steps in the negative direction within the same hardware event. See the
        :class:`~pywayland.protocol.wayland.WlPointer`.axis_value120 for more
        details.

        The value120 must not be zero.

        :param value120:
            rotation distance as fraction of 120
        :type value120:
            `ArgumentType.Int`
        """
        self._post_event(0, value120)

    @ZwpTabletPadDialV2.event(
        Argument(ArgumentType.Uint),
    )
    def frame(self, time: int) -> None:
        """End of a dial event sequence

        Indicates the end of a set of events that represent one logical
        hardware dial event. A client is expected to accumulate the data in all
        events within the frame before proceeding.

        All :class:`ZwpTabletPadDialV2` events before a
        :func:`ZwpTabletPadDialV2.frame()` event belong logically together.

        A :func:`ZwpTabletPadDialV2.frame()` event is sent for every logical
        event group, even if the group only contains a single
        :class:`ZwpTabletPadDialV2` event. Specifically, a client may get a
        sequence: delta, frame, delta, frame, etc.

        :param time:
            timestamp with millisecond granularity
        :type time:
            `ArgumentType.Uint`
        """
        self._post_event(1, time)


class ZwpTabletPadGroupV2Resource(Resource[ZwpTabletPadGroupV2]):
    interface = ZwpTabletPadGroupV2

    @ZwpTabletPadGroupV2.event(
        Argument(ArgumentType.Array),
    )
    def buttons(self, buttons: list) -> None:
        """Buttons announced

        Sent on :class:`ZwpTabletPadGroupV2` initialization to announce the
        available buttons in the group. Button indices start at 0, a button may
        only be in one group at a time.

        This event is first sent in the initial burst of events before the
        :func:`ZwpTabletPadGroupV2.done()` event.

        Some buttons are reserved by the compositor. These buttons may not be
        assigned to any :class:`ZwpTabletPadGroupV2`. Compositors may broadcast
        this event in the case of changes to the mapping of these reserved
        buttons. If the compositor happens to reserve all buttons in a group,
        this event will be sent with an empty array.

        :param buttons:
            buttons in this group
        :type buttons:
            `ArgumentType.Array`
        """
        self._post_event(0, buttons)

    @ZwpTabletPadGroupV2.event(
        Argument(ArgumentType.NewId, interface=ZwpTabletPadRingV2),
    )
    def ring(self, ring: ZwpTabletPadRingV2) -> None:
        """Ring announced

        Sent on :class:`ZwpTabletPadGroupV2` initialization to announce
        available rings. One event is sent for each ring available on this pad
        group.

        This event is sent in the initial burst of events before the
        :func:`ZwpTabletPadGroupV2.done()` event.

        :param ring:
        :type ring:
            :class:`~pywayland.protocol.tablet_v2.ZwpTabletPadRingV2`
        """
        self._post_event(1, ring)

    @ZwpTabletPadGroupV2.event(
        Argument(ArgumentType.NewId, interface=ZwpTabletPadStripV2),
    )
    def strip(self, strip: ZwpTabletPadStripV2) -> None:
        """Strip announced

        Sent on :class:`~pywayland.protocol.tablet_v2.ZwpTabletPadV2`
        initialization to announce available strips. One event is sent for each
        strip available on this pad group.

        This event is sent in the initial burst of events before the
        :func:`ZwpTabletPadGroupV2.done()` event.

        :param strip:
        :type strip:
            :class:`~pywayland.protocol.tablet_v2.ZwpTabletPadStripV2`
        """
        self._post_event(2, strip)

    @ZwpTabletPadGroupV2.event(
        Argument(ArgumentType.Uint),
    )
    def modes(self, modes: int) -> None:
        """Mode-switch ability announced

        Sent on :class:`ZwpTabletPadGroupV2` initialization to announce that
        the pad group may switch between modes. A client may use a mode to
        store a specific configuration for buttons, rings and strips and use
        the :func:`ZwpTabletPadGroupV2.mode_switch()` event to toggle between
        these configurations. Mode indices start at 0.

        Switching modes is compositor-dependent. See the
        :func:`ZwpTabletPadGroupV2.mode_switch()` event for more details.

        This event is sent in the initial burst of events before the
        :func:`ZwpTabletPadGroupV2.done()` event. This event is only sent when
        more than one mode is available.

        :param modes:
            the number of modes
        :type modes:
            `ArgumentType.Uint`
        """
        self._post_event(3, modes)

    @ZwpTabletPadGroupV2.event()
    def done(self) -> None:
        """Tablet group description events sequence complete

        This event is sent immediately to signal the end of the initial burst
        of descriptive events. A client may consider the static description of
        the tablet to be complete and finalize initialization of the tablet
        group.
        """
        self._post_event(4)

    @ZwpTabletPadGroupV2.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def mode_switch(self, time: int, serial: int, mode: int) -> None:
        """Mode switch event

        Notification that the mode was switched.

        A mode applies to all buttons, rings, strips and dials in a group
        simultaneously, but a client is not required to assign different
        actions for each mode. For example, a client may have mode-specific
        button mappings but map the ring to vertical scrolling in all modes.
        Mode indices start at 0.

        Switching modes is compositor-dependent. The compositor may provide
        visual cues to the user about the mode, e.g. by toggling LEDs on the
        tablet device. Mode-switching may be software-controlled or controlled
        by one or more physical buttons. For example, on a Wacom Intuos Pro,
        the button inside the ring may be assigned to switch between modes.

        The compositor will also send this event after
        :func:`ZwpTabletPadV2.enter()
        <pywayland.protocol.tablet_v2.ZwpTabletPadV2.enter>` on each group in
        order to notify of the current mode. Groups that only feature one mode
        will use mode=0 when emitting this event.

        If a button action in the new mode differs from the action in the
        previous mode, the client should immediately issue a
        :func:`ZwpTabletPadV2.set_feedback()
        <pywayland.protocol.tablet_v2.ZwpTabletPadV2.set_feedback>` request for
        each changed button.

        If a ring, strip or dial action in the new mode differs from the action
        in the previous mode, the client should immediately issue a
        zwp_tablet_ring_v2.set_feedback, zwp_tablet_strip_v2.set_feedback or
        zwp_tablet_dial_v2.set_feedback request for each changed ring, strip or
        dial.

        :param time:
            the time of the event with millisecond granularity
        :type time:
            `ArgumentType.Uint`
        :param serial:
        :type serial:
            `ArgumentType.Uint`
        :param mode:
            the new mode of the pad
        :type mode:
            `ArgumentType.Uint`
        """
        self._post_event(5, time, serial, mode)

    @ZwpTabletPadGroupV2.event(
        Argument(ArgumentType.NewId, interface=ZwpTabletPadDialV2),
        version=2,
    )
    def dial(self, dial: ZwpTabletPadDialV2) -> None:
        """Dial announced

        Sent on :class:`~pywayland.protocol.tablet_v2.ZwpTabletPadV2`
        initialization to announce available dials. One event is sent for each
        dial available on this pad group.

        This event is sent in the initial burst of events before the
        :func:`ZwpTabletPadGroupV2.done()` event.

        :param dial:
        :type dial:
            :class:`~pywayland.protocol.tablet_v2.ZwpTabletPadDialV2`
        """
        self._post_event(6, dial)


class ZwpTabletPadRingV2Resource(Resource[ZwpTabletPadRingV2]):
    interface = ZwpTabletPadRingV2

    @ZwpTabletPadRingV2.event(
        Argument(ArgumentType.Uint),
    )
    def source(self, source: int) -> None:
        """Ring event source

        Source information for ring events.

        This event does not occur on its own. It is sent before a
        :func:`ZwpTabletPadRingV2.frame()` event and carries the source
        information for all events within that frame.

        The source specifies how this event was generated. If the source is
        :func:`ZwpTabletPadRingV2.source()`.finger, a
        :func:`ZwpTabletPadRingV2.stop()` event will be sent when the user
        lifts the finger off the device.

        This event is optional. If the source is unknown for an interaction, no
        event is sent.

        :param source:
            the event source
        :type source:
            `ArgumentType.Uint`
        """
        self._post_event(0, source)

    @ZwpTabletPadRingV2.event(
        Argument(ArgumentType.Fixed),
    )
    def angle(self, degrees: float) -> None:
        """Angle changed

        Sent whenever the angle on a ring changes.

        The angle is provided in degrees clockwise from the logical north of
        the ring in the pad's current rotation.

        :param degrees:
            the current angle in degrees
        :type degrees:
            `ArgumentType.Fixed`
        """
        self._post_event(1, degrees)

    @ZwpTabletPadRingV2.event()
    def stop(self) -> None:
        """Interaction stopped

        Stop notification for ring events.

        For some :func:`ZwpTabletPadRingV2.source()` types, a
        :func:`ZwpTabletPadRingV2.stop()` event is sent to notify a client that
        the interaction with the ring has terminated. This enables the client
        to implement kinetic scrolling. See the
        :func:`ZwpTabletPadRingV2.source()` documentation for information on
        when this event may be generated.

        Any :func:`ZwpTabletPadRingV2.angle()` events with the same source
        after this event should be considered as the start of a new
        interaction.
        """
        self._post_event(2)

    @ZwpTabletPadRingV2.event(
        Argument(ArgumentType.Uint),
    )
    def frame(self, time: int) -> None:
        """End of a ring event sequence

        Indicates the end of a set of ring events that logically belong
        together. A client is expected to accumulate the data in all events
        within the frame before proceeding.

        All :class:`ZwpTabletPadRingV2` events before a
        :func:`ZwpTabletPadRingV2.frame()` event belong logically together. For
        example, on termination of a finger interaction on a ring the
        compositor will send a :func:`ZwpTabletPadRingV2.source()` event, a
        :func:`ZwpTabletPadRingV2.stop()` event and a
        :func:`ZwpTabletPadRingV2.frame()` event.

        A :func:`ZwpTabletPadRingV2.frame()` event is sent for every logical
        event group, even if the group only contains a single
        :class:`ZwpTabletPadRingV2` event. Specifically, a client may get a
        sequence: angle, frame, angle, frame, etc.

        :param time:
            timestamp with millisecond granularity
        :type time:
            `ArgumentType.Uint`
        """
        self._post_event(3, time)


class ZwpTabletPadStripV2Resource(Resource[ZwpTabletPadStripV2]):
    interface = ZwpTabletPadStripV2

    @ZwpTabletPadStripV2.event(
        Argument(ArgumentType.Uint),
    )
    def source(self, source: int) -> None:
        """Strip event source

        Source information for strip events.

        This event does not occur on its own. It is sent before a
        :func:`ZwpTabletPadStripV2.frame()` event and carries the source
        information for all events within that frame.

        The source specifies how this event was generated. If the source is
        :func:`ZwpTabletPadStripV2.source()`.finger, a
        :func:`ZwpTabletPadStripV2.stop()` event will be sent when the user
        lifts their finger off the device.

        This event is optional. If the source is unknown for an interaction, no
        event is sent.

        :param source:
            the event source
        :type source:
            `ArgumentType.Uint`
        """
        self._post_event(0, source)

    @ZwpTabletPadStripV2.event(
        Argument(ArgumentType.Uint),
    )
    def position(self, position: int) -> None:
        """Position changed

        Sent whenever the position on a strip changes.

        The position is normalized to a range of [0, 65535], the 0-value
        represents the top-most and/or left-most position of the strip in the
        pad's current rotation.

        :param position:
            the current position
        :type position:
            `ArgumentType.Uint`
        """
        self._post_event(1, position)

    @ZwpTabletPadStripV2.event()
    def stop(self) -> None:
        """Interaction stopped

        Stop notification for strip events.

        For some :func:`ZwpTabletPadStripV2.source()` types, a
        :func:`ZwpTabletPadStripV2.stop()` event is sent to notify a client
        that the interaction with the strip has terminated. This enables the
        client to implement kinetic scrolling. See the
        :func:`ZwpTabletPadStripV2.source()` documentation for information on
        when this event may be generated.

        Any :func:`ZwpTabletPadStripV2.position()` events with the same source
        after this event should be considered as the start of a new
        interaction.
        """
        self._post_event(2)

    @ZwpTabletPadStripV2.event(
        Argument(ArgumentType.Uint),
    )
    def frame(self, time: int) -> None:
        """End of a strip event sequence

        Indicates the end of a set of events that represent one logical
        hardware strip event. A client is expected to accumulate the data in
        all events within the frame before proceeding.

        All :class:`ZwpTabletPadStripV2` events before a
        :func:`ZwpTabletPadStripV2.frame()` event belong logically together.
        For example, on termination of a finger interaction on a strip the
        compositor will send a :func:`ZwpTabletPadStripV2.source()` event, a
        :func:`ZwpTabletPadStripV2.stop()` event and a
        :func:`ZwpTabletPadStripV2.frame()` event.

        A :func:`ZwpTabletPadStripV2.frame()` event is sent for every logical
        event group, even if the group only contains a single
        :class:`ZwpTabletPadStripV2` event. Specifically, a client may get a
        sequence: position, frame, position, frame, etc.

        :param time:
            timestamp with millisecond granularity
        :type time:
            `ArgumentType.Uint`
        """
        self._post_event(3, time)


class ZwpTabletPadV2Resource(Resource[ZwpTabletPadV2]):
    interface = ZwpTabletPadV2

    @ZwpTabletPadV2.event(
        Argument(ArgumentType.NewId, interface=ZwpTabletPadGroupV2),
    )
    def group(self, pad_group: ZwpTabletPadGroupV2) -> None:
        """Group announced

        Sent on :class:`ZwpTabletPadV2` initialization to announce available
        groups. One event is sent for each pad group available.

        This event is sent in the initial burst of events before the
        :func:`ZwpTabletPadV2.done()` event. At least one group will be
        announced.

        :param pad_group:
        :type pad_group:
            :class:`~pywayland.protocol.tablet_v2.ZwpTabletPadGroupV2`
        """
        self._post_event(0, pad_group)

    @ZwpTabletPadV2.event(
        Argument(ArgumentType.String),
    )
    def path(self, path: str) -> None:
        """Path to the device

        A system-specific device path that indicates which device is behind
        this :class:`ZwpTabletPadV2`. This information may be used to gather
        additional information about the device, e.g. through libwacom.

        The format of the path is unspecified, it may be a device node, a sysfs
        path, or some other identifier. It is up to the client to identify the
        string provided.

        This event is sent in the initial burst of events before the
        :func:`ZwpTabletPadV2.done()` event.

        :param path:
            path to local device
        :type path:
            `ArgumentType.String`
        """
        self._post_event(1, path)

    @ZwpTabletPadV2.event(
        Argument(ArgumentType.Uint),
    )
    def buttons(self, buttons: int) -> None:
        """Buttons announced

        Sent on :class:`ZwpTabletPadV2` initialization to announce the
        available buttons.

        This event is sent in the initial burst of events before the
        :func:`ZwpTabletPadV2.done()` event. This event is only sent when at
        least one button is available.

        :param buttons:
            the number of buttons
        :type buttons:
            `ArgumentType.Uint`
        """
        self._post_event(2, buttons)

    @ZwpTabletPadV2.event()
    def done(self) -> None:
        """Pad description event sequence complete

        This event signals the end of the initial burst of descriptive events.
        A client may consider the static description of the pad to be complete
        and finalize initialization of the pad.
        """
        self._post_event(3)

    @ZwpTabletPadV2.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def button(self, time: int, button: int, state: int) -> None:
        """Physical button state

        Sent whenever the physical state of a button changes.

        :param time:
            the time of the event with millisecond granularity
        :type time:
            `ArgumentType.Uint`
        :param button:
            the index of the button that changed state
        :type button:
            `ArgumentType.Uint`
        :param state:
        :type state:
            `ArgumentType.Uint`
        """
        self._post_event(4, time, button, state)

    @ZwpTabletPadV2.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Object, interface=ZwpTabletV2),
        Argument(ArgumentType.Object, interface=WlSurface),
    )
    def enter(self, serial: int, tablet: ZwpTabletV2, surface: WlSurface) -> None:
        """Enter event

        Notification that this pad is focused on the specified surface.

        :param serial:
            serial number of the enter event
        :type serial:
            `ArgumentType.Uint`
        :param tablet:
            the tablet the pad is attached to
        :type tablet:
            :class:`~pywayland.protocol.tablet_v2.ZwpTabletV2`
        :param surface:
            surface the pad is focused on
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        """
        self._post_event(5, serial, tablet, surface)

    @ZwpTabletPadV2.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Object, interface=WlSurface),
    )
    def leave(self, serial: int, surface: WlSurface) -> None:
        """Leave event

        Notification that this pad is no longer focused on the specified
        surface.

        :param serial:
            serial number of the leave event
        :type serial:
            `ArgumentType.Uint`
        :param surface:
            surface the pad is no longer focused on
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        """
        self._post_event(6, serial, surface)

    @ZwpTabletPadV2.event()
    def removed(self) -> None:
        """Pad removed event

        Sent when the pad has been removed from the system. When a tablet is
        removed its pad(s) will be removed too.

        When this event is received, the client must destroy all rings, strips
        and groups that were offered by this pad, and issue
        :func:`ZwpTabletPadV2.destroy()` the pad itself.
        """
        self._post_event(7)


class ZwpTabletSeatV2Resource(Resource[ZwpTabletSeatV2]):
    interface = ZwpTabletSeatV2

    @ZwpTabletSeatV2.event(
        Argument(ArgumentType.NewId, interface=ZwpTabletV2),
    )
    def tablet_added(self, id: ZwpTabletV2) -> None:
        """New device notification

        This event is sent whenever a new tablet becomes available on this
        seat. This event only provides the object id of the tablet, any static
        information about the tablet (device name, vid/pid, etc.) is sent
        through the :class:`~pywayland.protocol.tablet_v2.ZwpTabletV2`
        interface.

        :param id:
            the newly added graphics tablet
        :type id:
            :class:`~pywayland.protocol.tablet_v2.ZwpTabletV2`
        """
        self._post_event(0, id)

    @ZwpTabletSeatV2.event(
        Argument(ArgumentType.NewId, interface=ZwpTabletToolV2),
    )
    def tool_added(self, id: ZwpTabletToolV2) -> None:
        """A new tool has been used with a tablet

        This event is sent whenever a tool that has not previously been used
        with a tablet comes into use. This event only provides the object id of
        the tool; any static information about the tool (capabilities, type,
        etc.) is sent through the
        :class:`~pywayland.protocol.tablet_v2.ZwpTabletToolV2` interface.

        :param id:
            the newly added tablet tool
        :type id:
            :class:`~pywayland.protocol.tablet_v2.ZwpTabletToolV2`
        """
        self._post_event(1, id)

    @ZwpTabletSeatV2.event(
        Argument(ArgumentType.NewId, interface=ZwpTabletPadV2),
    )
    def pad_added(self, id: ZwpTabletPadV2) -> None:
        """New pad notification

        This event is sent whenever a new pad is known to the system.
        Typically, pads are physically attached to tablets and a pad_added
        event is sent immediately after the
        :func:`ZwpTabletSeatV2.tablet_added()`. However, some standalone pad
        devices logically attach to tablets at runtime, and the client must
        wait for :func:`ZwpTabletPadV2.enter()
        <pywayland.protocol.tablet_v2.ZwpTabletPadV2.enter>` to know the tablet
        a pad is attached to.

        This event only provides the object id of the pad. All further features
        (buttons, strips, rings) are sent through the
        :class:`~pywayland.protocol.tablet_v2.ZwpTabletPadV2` interface.

        :param id:
            the newly added pad
        :type id:
            :class:`~pywayland.protocol.tablet_v2.ZwpTabletPadV2`
        """
        self._post_event(2, id)


class ZwpTabletToolV2Resource(Resource[ZwpTabletToolV2]):
    interface = ZwpTabletToolV2

    @ZwpTabletToolV2.event(
        Argument(ArgumentType.Uint),
    )
    def type(self, tool_type: int) -> None:
        """Tool type

        The tool type is the high-level type of the tool and usually decides
        the interaction expected from this tool.

        This event is sent in the initial burst of events before the
        :func:`ZwpTabletToolV2.done()` event.

        :param tool_type:
            the physical tool type
        :type tool_type:
            `ArgumentType.Uint`
        """
        self._post_event(0, tool_type)

    @ZwpTabletToolV2.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def hardware_serial(self, hardware_serial_hi: int, hardware_serial_lo: int) -> None:
        """Unique hardware serial number of the tool

        If the physical tool can be identified by a unique 64-bit serial
        number, this event notifies the client of this serial number.

        If multiple tablets are available in the same seat and the tool is
        uniquely identifiable by the serial number, that tool may move between
        tablets.

        Otherwise, if the tool has no serial number and this event is missing,
        the tool is tied to the tablet it first comes into proximity with. Even
        if the physical tool is used on multiple tablets, separate
        :class:`ZwpTabletToolV2` objects will be created, one per tablet.

        This event is sent in the initial burst of events before the
        :func:`ZwpTabletToolV2.done()` event.

        :param hardware_serial_hi:
            the unique serial number of the tool, most significant bits
        :type hardware_serial_hi:
            `ArgumentType.Uint`
        :param hardware_serial_lo:
            the unique serial number of the tool, least significant bits
        :type hardware_serial_lo:
            `ArgumentType.Uint`
        """
        self._post_event(1, hardware_serial_hi, hardware_serial_lo)

    @ZwpTabletToolV2.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def hardware_id_wacom(self, hardware_id_hi: int, hardware_id_lo: int) -> None:
        """Hardware id notification in wacom's format

        This event notifies the client of a hardware id available on this tool.

        The hardware id is a device-specific 64-bit id that provides extra
        information about the tool in use, beyond the wl_tool.type enumeration.
        The format of the id is specific to tablets made by Wacom Inc. For
        example, the hardware id of a Wacom Grip Pen (a stylus) is 0x802.

        This event is sent in the initial burst of events before the
        :func:`ZwpTabletToolV2.done()` event.

        :param hardware_id_hi:
            the hardware id, most significant bits
        :type hardware_id_hi:
            `ArgumentType.Uint`
        :param hardware_id_lo:
            the hardware id, least significant bits
        :type hardware_id_lo:
            `ArgumentType.Uint`
        """
        self._post_event(2, hardware_id_hi, hardware_id_lo)

    @ZwpTabletToolV2.event(
        Argument(ArgumentType.Uint),
    )
    def capability(self, capability: int) -> None:
        """Tool capability notification

        This event notifies the client of any capabilities of this tool, beyond
        the main set of x/y axes and tip up/down detection.

        One event is sent for each extra capability available on this tool.

        This event is sent in the initial burst of events before the
        :func:`ZwpTabletToolV2.done()` event.

        :param capability:
            the capability
        :type capability:
            `ArgumentType.Uint`
        """
        self._post_event(3, capability)

    @ZwpTabletToolV2.event()
    def done(self) -> None:
        """Tool description events sequence complete

        This event signals the end of the initial burst of descriptive events.
        A client may consider the static description of the tool to be complete
        and finalize initialization of the tool.
        """
        self._post_event(4)

    @ZwpTabletToolV2.event()
    def removed(self) -> None:
        """Tool removed

        This event is sent when the tool is removed from the system and will
        send no further events. Should the physical tool come back into
        proximity later, a new :class:`ZwpTabletToolV2` object will be created.

        It is compositor-dependent when a tool is removed. A compositor may
        remove a tool on proximity out, tablet removal or any other reason. A
        compositor may also keep a tool alive until shutdown.

        If the tool is currently in proximity, a proximity_out event will be
        sent before the removed event. See
        :func:`ZwpTabletToolV2.proximity_out()` for the handling of any buttons
        logically down.

        When this event is received, the client must
        :func:`ZwpTabletToolV2.destroy()` the object.
        """
        self._post_event(5)

    @ZwpTabletToolV2.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Object, interface=ZwpTabletV2),
        Argument(ArgumentType.Object, interface=WlSurface),
    )
    def proximity_in(self, serial: int, tablet: ZwpTabletV2, surface: WlSurface) -> None:
        """Proximity in event

        Notification that this tool is focused on a certain surface.

        This event can be received when the tool has moved from one surface to
        another, or when the tool has come back into proximity above the
        surface.

        If any button is logically down when the tool comes into proximity, the
        respective button event is sent after the proximity_in event but within
        the same frame as the proximity_in event.

        :param serial:
        :type serial:
            `ArgumentType.Uint`
        :param tablet:
            The tablet the tool is in proximity of
        :type tablet:
            :class:`~pywayland.protocol.tablet_v2.ZwpTabletV2`
        :param surface:
            The current surface the tablet tool is over
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        """
        self._post_event(6, serial, tablet, surface)

    @ZwpTabletToolV2.event()
    def proximity_out(self) -> None:
        """Proximity out event

        Notification that this tool has either left proximity, or is no longer
        focused on a certain surface.

        When the tablet tool leaves proximity of the tablet, button release
        events are sent for each button that was held down at the time of
        leaving proximity. These events are sent before the proximity_out event
        but within the same :func:`ZwpTabletV2.frame()
        <pywayland.protocol.tablet_v2.ZwpTabletV2.frame>`.

        If the tool stays within proximity of the tablet, but the focus changes
        from one surface to another, a button release event may not be sent
        until the button is actually released or the tool leaves the proximity
        of the tablet.
        """
        self._post_event(7)

    @ZwpTabletToolV2.event(
        Argument(ArgumentType.Uint),
    )
    def down(self, serial: int) -> None:
        """Tablet tool is making contact

        Sent whenever the tablet tool comes in contact with the surface of the
        tablet.

        If the tool is already in contact with the tablet when entering the
        input region, the client owning said region will receive a
        :func:`ZwpTabletV2.proximity_in()
        <pywayland.protocol.tablet_v2.ZwpTabletV2.proximity_in>` event,
        followed by a :func:`ZwpTabletV2.down()
        <pywayland.protocol.tablet_v2.ZwpTabletV2.down>` event and a
        :func:`ZwpTabletV2.frame()
        <pywayland.protocol.tablet_v2.ZwpTabletV2.frame>` event.

        Note that this event describes logical contact, not physical contact.
        On some devices, a compositor may not consider a tool in logical
        contact until a minimum physical pressure threshold is exceeded.

        :param serial:
        :type serial:
            `ArgumentType.Uint`
        """
        self._post_event(8, serial)

    @ZwpTabletToolV2.event()
    def up(self) -> None:
        """Tablet tool is no longer making contact

        Sent whenever the tablet tool stops making contact with the surface of
        the tablet, or when the tablet tool moves out of the input region and
        the compositor grab (if any) is dismissed.

        If the tablet tool moves out of the input region while in contact with
        the surface of the tablet and the compositor does not have an ongoing
        grab on the surface, the client owning said region will receive a
        :func:`ZwpTabletV2.up() <pywayland.protocol.tablet_v2.ZwpTabletV2.up>`
        event, followed by a :func:`ZwpTabletV2.proximity_out()
        <pywayland.protocol.tablet_v2.ZwpTabletV2.proximity_out>` event and a
        :func:`ZwpTabletV2.frame()
        <pywayland.protocol.tablet_v2.ZwpTabletV2.frame>` event. If the
        compositor has an ongoing grab on this device, this event sequence is
        sent whenever the grab is dismissed in the future.

        Note that this event describes logical contact, not physical contact.
        On some devices, a compositor may not consider a tool out of logical
        contact until physical pressure falls below a specific threshold.
        """
        self._post_event(9)

    @ZwpTabletToolV2.event(
        Argument(ArgumentType.Fixed),
        Argument(ArgumentType.Fixed),
    )
    def motion(self, x: float, y: float) -> None:
        """Motion event

        Sent whenever a tablet tool moves.

        :param x:
            surface-local x coordinate
        :type x:
            `ArgumentType.Fixed`
        :param y:
            surface-local y coordinate
        :type y:
            `ArgumentType.Fixed`
        """
        self._post_event(10, x, y)

    @ZwpTabletToolV2.event(
        Argument(ArgumentType.Uint),
    )
    def pressure(self, pressure: int) -> None:
        """Pressure change event

        Sent whenever the pressure axis on a tool changes. The value of this
        event is normalized to a value between 0 and 65535.

        Note that pressure may be nonzero even when a tool is not in logical
        contact. See the down and up events for more details.

        :param pressure:
            The current pressure value
        :type pressure:
            `ArgumentType.Uint`
        """
        self._post_event(11, pressure)

    @ZwpTabletToolV2.event(
        Argument(ArgumentType.Uint),
    )
    def distance(self, distance: int) -> None:
        """Distance change event

        Sent whenever the distance axis on a tool changes. The value of this
        event is normalized to a value between 0 and 65535.

        Note that distance may be nonzero even when a tool is not in logical
        contact. See the down and up events for more details.

        :param distance:
            The current distance value
        :type distance:
            `ArgumentType.Uint`
        """
        self._post_event(12, distance)

    @ZwpTabletToolV2.event(
        Argument(ArgumentType.Fixed),
        Argument(ArgumentType.Fixed),
    )
    def tilt(self, tilt_x: float, tilt_y: float) -> None:
        """Tilt change event

        Sent whenever one or both of the tilt axes on a tool change. Each tilt
        value is in degrees, relative to the z-axis of the tablet. The angle is
        positive when the top of a tool tilts along the positive x or y axis.

        :param tilt_x:
            The current value of the X tilt axis
        :type tilt_x:
            `ArgumentType.Fixed`
        :param tilt_y:
            The current value of the Y tilt axis
        :type tilt_y:
            `ArgumentType.Fixed`
        """
        self._post_event(13, tilt_x, tilt_y)

    @ZwpTabletToolV2.event(
        Argument(ArgumentType.Fixed),
    )
    def rotation(self, degrees: float) -> None:
        """Z-rotation change event

        Sent whenever the z-rotation axis on the tool changes. The rotation
        value is in degrees clockwise from the tool's logical neutral position.

        :param degrees:
            The current rotation of the Z axis
        :type degrees:
            `ArgumentType.Fixed`
        """
        self._post_event(14, degrees)

    @ZwpTabletToolV2.event(
        Argument(ArgumentType.Int),
    )
    def slider(self, position: int) -> None:
        """Slider position change event

        Sent whenever the slider position on the tool changes. The value is
        normalized between -65535 and 65535, with 0 as the logical neutral
        position of the slider.

        The slider is available on e.g. the Wacom Airbrush tool.

        :param position:
            The current position of slider
        :type position:
            `ArgumentType.Int`
        """
        self._post_event(15, position)

    @ZwpTabletToolV2.event(
        Argument(ArgumentType.Fixed),
        Argument(ArgumentType.Int),
    )
    def wheel(self, degrees: float, clicks: int) -> None:
        """Wheel delta event

        Sent whenever the wheel on the tool emits an event. This event contains
        two values for the same axis change. The degrees value is in the same
        orientation as the :func:`WlPointer.vertical_scroll()
        <pywayland.protocol.wayland.WlPointer.vertical_scroll>` axis. The
        clicks value is in discrete logical clicks of the mouse wheel. This
        value may be zero if the movement of the wheel was less than one
        logical click.

        Clients should choose either value and avoid mixing degrees and clicks.
        The compositor may accumulate values smaller than a logical click and
        emulate click events when a certain threshold is met. Thus,
        :func:`ZwpTabletToolV2.wheel()` events with non-zero clicks values may
        have different degrees values.

        :param degrees:
            The wheel delta in degrees
        :type degrees:
            `ArgumentType.Fixed`
        :param clicks:
            The wheel delta in discrete clicks
        :type clicks:
            `ArgumentType.Int`
        """
        self._post_event(16, degrees, clicks)

    @ZwpTabletToolV2.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def button(self, serial: int, button: int, state: int) -> None:
        """Button event

        Sent whenever a button on the tool is pressed or released.

        If a button is held down when the tool moves in or out of proximity,
        button events are generated by the compositor. See
        :func:`ZwpTabletToolV2.proximity_in()` and
        :func:`ZwpTabletToolV2.proximity_out()` for details.

        :param serial:
        :type serial:
            `ArgumentType.Uint`
        :param button:
            The button whose state has changed
        :type button:
            `ArgumentType.Uint`
        :param state:
            Whether the button was pressed or released
        :type state:
            `ArgumentType.Uint`
        """
        self._post_event(17, serial, button, state)

    @ZwpTabletToolV2.event(
        Argument(ArgumentType.Uint),
    )
    def frame(self, time: int) -> None:
        """Frame event

        Marks the end of a series of axis and/or button updates from the
        tablet. The Wayland protocol requires axis updates to be sent
        sequentially, however all events within a frame should be considered
        one hardware event.

        :param time:
            The time of the event with millisecond granularity
        :type time:
            `ArgumentType.Uint`
        """
        self._post_event(18, time)


class ZwpTabletV2Resource(Resource[ZwpTabletV2]):
    interface = ZwpTabletV2

    @ZwpTabletV2.event(
        Argument(ArgumentType.String),
    )
    def name(self, name: str) -> None:
        """Tablet device name

        A descriptive name for the tablet device.

        If the device has no descriptive name, this event is not sent.

        This event is sent in the initial burst of events before the
        :func:`ZwpTabletV2.done()` event.

        :param name:
            the device name
        :type name:
            `ArgumentType.String`
        """
        self._post_event(0, name)

    @ZwpTabletV2.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def id(self, vid: int, pid: int) -> None:
        """Tablet device vendor/product id

        The vendor and product IDs for the tablet device.

        The interpretation of the id depends on the
        :func:`ZwpTabletV2.bustype()`. Prior to version v2 of this protocol,
        the id was implied to be a USB vendor and product ID. If no
        :func:`ZwpTabletV2.bustype()` is sent, the ID is to be interpreted as
        USB vendor and product ID.

        If the device has no vendor/product ID, this event is not sent. This
        can happen for virtual devices or non-USB devices, for instance.

        This event is sent in the initial burst of events before the
        :func:`ZwpTabletV2.done()` event.

        :param vid:
            vendor id
        :type vid:
            `ArgumentType.Uint`
        :param pid:
            product id
        :type pid:
            `ArgumentType.Uint`
        """
        self._post_event(1, vid, pid)

    @ZwpTabletV2.event(
        Argument(ArgumentType.String),
    )
    def path(self, path: str) -> None:
        """Path to the device

        A system-specific device path that indicates which device is behind
        this :class:`ZwpTabletV2`. This information may be used to gather
        additional information about the device, e.g. through libwacom.

        A device may have more than one device path. If so, multiple
        :func:`ZwpTabletV2.path()` events are sent. A device may be emulated
        and not have a device path, and in that case this event will not be
        sent.

        The format of the path is unspecified, it may be a device node, a sysfs
        path, or some other identifier. It is up to the client to identify the
        string provided.

        This event is sent in the initial burst of events before the
        :func:`ZwpTabletV2.done()` event.

        :param path:
            path to local device
        :type path:
            `ArgumentType.String`
        """
        self._post_event(2, path)

    @ZwpTabletV2.event()
    def done(self) -> None:
        """Tablet description events sequence complete

        This event is sent immediately to signal the end of the initial burst
        of descriptive events. A client may consider the static description of
        the tablet to be complete and finalize initialization of the tablet.
        """
        self._post_event(3)

    @ZwpTabletV2.event()
    def removed(self) -> None:
        """Tablet removed event

        Sent when the tablet has been removed from the system. When a tablet is
        removed, some tools may be removed.

        When this event is received, the client must
        :func:`ZwpTabletV2.destroy()` the object.
        """
        self._post_event(4)

    @ZwpTabletV2.event(
        Argument(ArgumentType.Uint),
        version=2,
    )
    def bustype(self, bustype: int) -> None:
        """Tablet device bus type

        The bustype argument is one of the BUS_ defines in the Linux kernel's
        linux/input.h

        If the device has no known bustype or the bustype cannot be queried,
        this event is not sent.

        This event is sent in the initial burst of events before the
        :func:`ZwpTabletV2.done()` event.

        :param bustype:
            bus type
        :type bustype:
            `ArgumentType.Uint`
        """
        self._post_event(5, bustype)


class ZwpTabletManagerV2Proxy(Proxy[ZwpTabletManagerV2]):
    interface = ZwpTabletManagerV2

    @ZwpTabletManagerV2.request(
        Argument(ArgumentType.NewId, interface=ZwpTabletSeatV2),
        Argument(ArgumentType.Object, interface=WlSeat),
    )
    def get_tablet_seat(self, seat: WlSeatProxy) -> ZwpTabletSeatV2Proxy:
        """Get the tablet seat

        Get the :class:`~pywayland.protocol.tablet_v2.ZwpTabletSeatV2` object
        for the given seat. This object provides access to all graphics tablets
        in this seat.

        :param seat:
            The :class:`~pywayland.protocol.wayland.WlSeat` object to retrieve
            the tablets for
        :type seat:
            :class:`~pywayland.protocol.wayland.WlSeat`
        :returns:
            :class:`~pywayland.protocol.tablet_v2.ZwpTabletSeatV2`
        """
        tablet_seat = self._marshal_constructor(0, ZwpTabletSeatV2, seat)
        return tablet_seat

    @ZwpTabletManagerV2.request()
    def destroy(self) -> None:
        """Release the memory for the tablet manager object

        Destroy the :class:`ZwpTabletManagerV2` object. Objects created from
        this object are unaffected and should be destroyed separately.
        """
        self._marshal(1)
        self._destroy()


class ZwpTabletPadDialV2Proxy(Proxy[ZwpTabletPadDialV2]):
    interface = ZwpTabletPadDialV2

    @ZwpTabletPadDialV2.request(
        Argument(ArgumentType.String),
        Argument(ArgumentType.Uint),
    )
    def set_feedback(self, description: str, serial: int) -> None:
        """Set compositor feedback

        Requests the compositor to use the provided feedback string associated
        with this dial. This request should be issued immediately after a
        :func:`ZwpTabletPadGroupV2.mode_switch()
        <pywayland.protocol.tablet_v2.ZwpTabletPadGroupV2.mode_switch>` event
        from the corresponding group is received, or whenever the dial is
        mapped to a different action. See
        :func:`ZwpTabletPadGroupV2.mode_switch()
        <pywayland.protocol.tablet_v2.ZwpTabletPadGroupV2.mode_switch>` for
        more details.

        Clients are encouraged to provide context-aware descriptions for the
        actions associated with the dial, and compositors may use this
        information to offer visual feedback about the button layout (eg. on-
        screen displays).

        The provided string 'description' is a UTF-8 encoded string to be
        associated with this ring, and is considered user-visible; general
        internationalization rules apply.

        The serial argument will be that of the last
        :func:`ZwpTabletPadGroupV2.mode_switch()
        <pywayland.protocol.tablet_v2.ZwpTabletPadGroupV2.mode_switch>` event
        received for the group of this dial. Requests providing other serials
        than the most recent one will be ignored.

        :param description:
            dial description
        :type description:
            `ArgumentType.String`
        :param serial:
            serial of the mode switch event
        :type serial:
            `ArgumentType.Uint`
        """
        self._marshal(0, description, serial)

    @ZwpTabletPadDialV2.request()
    def destroy(self) -> None:
        """Destroy the dial object

        This destroys the client's resource for this dial object.
        """
        self._marshal(1)
        self._destroy()


class ZwpTabletPadGroupV2Proxy(Proxy[ZwpTabletPadGroupV2]):
    interface = ZwpTabletPadGroupV2

    @ZwpTabletPadGroupV2.request()
    def destroy(self) -> None:
        """Destroy the pad object

        Destroy the :class:`ZwpTabletPadGroupV2` object. Objects created from
        this object are unaffected and should be destroyed separately.
        """
        self._marshal(0)
        self._destroy()


class ZwpTabletPadRingV2Proxy(Proxy[ZwpTabletPadRingV2]):
    interface = ZwpTabletPadRingV2

    @ZwpTabletPadRingV2.request(
        Argument(ArgumentType.String),
        Argument(ArgumentType.Uint),
    )
    def set_feedback(self, description: str, serial: int) -> None:
        """Set compositor feedback

        Request that the compositor use the provided feedback string associated
        with this ring. This request should be issued immediately after a
        :func:`ZwpTabletPadGroupV2.mode_switch()
        <pywayland.protocol.tablet_v2.ZwpTabletPadGroupV2.mode_switch>` event
        from the corresponding group is received, or whenever the ring is
        mapped to a different action. See
        :func:`ZwpTabletPadGroupV2.mode_switch()
        <pywayland.protocol.tablet_v2.ZwpTabletPadGroupV2.mode_switch>` for
        more details.

        Clients are encouraged to provide context-aware descriptions for the
        actions associated with the ring; compositors may use this information
        to offer visual feedback about the button layout (eg. on-screen
        displays).

        The provided string 'description' is a UTF-8 encoded string to be
        associated with this ring, and is considered user-visible; general
        internationalization rules apply.

        The serial argument will be that of the last
        :func:`ZwpTabletPadGroupV2.mode_switch()
        <pywayland.protocol.tablet_v2.ZwpTabletPadGroupV2.mode_switch>` event
        received for the group of this ring. Requests providing other serials
        than the most recent one will be ignored.

        :param description:
            ring description
        :type description:
            `ArgumentType.String`
        :param serial:
            serial of the mode switch event
        :type serial:
            `ArgumentType.Uint`
        """
        self._marshal(0, description, serial)

    @ZwpTabletPadRingV2.request()
    def destroy(self) -> None:
        """Destroy the ring object

        This destroys the client's resource for this ring object.
        """
        self._marshal(1)
        self._destroy()


class ZwpTabletPadStripV2Proxy(Proxy[ZwpTabletPadStripV2]):
    interface = ZwpTabletPadStripV2

    @ZwpTabletPadStripV2.request(
        Argument(ArgumentType.String),
        Argument(ArgumentType.Uint),
    )
    def set_feedback(self, description: str, serial: int) -> None:
        """Set compositor feedback

        Requests the compositor to use the provided feedback string associated
        with this strip. This request should be issued immediately after a
        :func:`ZwpTabletPadGroupV2.mode_switch()
        <pywayland.protocol.tablet_v2.ZwpTabletPadGroupV2.mode_switch>` event
        from the corresponding group is received, or whenever the strip is
        mapped to a different action. See
        :func:`ZwpTabletPadGroupV2.mode_switch()
        <pywayland.protocol.tablet_v2.ZwpTabletPadGroupV2.mode_switch>` for
        more details.

        Clients are encouraged to provide context-aware descriptions for the
        actions associated with the strip, and compositors may use this
        information to offer visual feedback about the button layout (eg. on-
        screen displays).

        The provided string 'description' is a UTF-8 encoded string to be
        associated with this ring, and is considered user-visible; general
        internationalization rules apply.

        The serial argument will be that of the last
        :func:`ZwpTabletPadGroupV2.mode_switch()
        <pywayland.protocol.tablet_v2.ZwpTabletPadGroupV2.mode_switch>` event
        received for the group of this strip. Requests providing other serials
        than the most recent one will be ignored.

        :param description:
            strip description
        :type description:
            `ArgumentType.String`
        :param serial:
            serial of the mode switch event
        :type serial:
            `ArgumentType.Uint`
        """
        self._marshal(0, description, serial)

    @ZwpTabletPadStripV2.request()
    def destroy(self) -> None:
        """Destroy the strip object

        This destroys the client's resource for this strip object.
        """
        self._marshal(1)
        self._destroy()


class ZwpTabletPadV2Proxy(Proxy[ZwpTabletPadV2]):
    interface = ZwpTabletPadV2

    @ZwpTabletPadV2.request(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.String),
        Argument(ArgumentType.Uint),
    )
    def set_feedback(self, button: int, description: str, serial: int) -> None:
        """Set compositor feedback

        Requests the compositor to use the provided feedback string associated
        with this button. This request should be issued immediately after a
        :func:`ZwpTabletPadGroupV2.mode_switch()
        <pywayland.protocol.tablet_v2.ZwpTabletPadGroupV2.mode_switch>` event
        from the corresponding group is received, or whenever a button is
        mapped to a different action. See
        :func:`ZwpTabletPadGroupV2.mode_switch()
        <pywayland.protocol.tablet_v2.ZwpTabletPadGroupV2.mode_switch>` for
        more details.

        Clients are encouraged to provide context-aware descriptions for the
        actions associated with each button, and compositors may use this
        information to offer visual feedback on the button layout (e.g. on-
        screen displays).

        Button indices start at 0. Setting the feedback string on a button that
        is reserved by the compositor (i.e. not belonging to any
        :class:`~pywayland.protocol.tablet_v2.ZwpTabletPadGroupV2`) does not
        generate an error but the compositor is free to ignore the request.

        The provided string 'description' is a UTF-8 encoded string to be
        associated with this ring, and is considered user-visible; general
        internationalization rules apply.

        The serial argument will be that of the last
        :func:`ZwpTabletPadGroupV2.mode_switch()
        <pywayland.protocol.tablet_v2.ZwpTabletPadGroupV2.mode_switch>` event
        received for the group of this button. Requests providing other serials
        than the most recent one will be ignored.

        :param button:
            button index
        :type button:
            `ArgumentType.Uint`
        :param description:
            button description
        :type description:
            `ArgumentType.String`
        :param serial:
            serial of the mode switch event
        :type serial:
            `ArgumentType.Uint`
        """
        self._marshal(0, button, description, serial)

    @ZwpTabletPadV2.request()
    def destroy(self) -> None:
        """Destroy the pad object

        Destroy the :class:`ZwpTabletPadV2` object. Objects created from this
        object are unaffected and should be destroyed separately.
        """
        self._marshal(1)
        self._destroy()


class ZwpTabletSeatV2Proxy(Proxy[ZwpTabletSeatV2]):
    interface = ZwpTabletSeatV2

    @ZwpTabletSeatV2.request()
    def destroy(self) -> None:
        """Release the memory for the tablet seat object

        Destroy the :class:`ZwpTabletSeatV2` object. Objects created from this
        object are unaffected and should be destroyed separately.
        """
        self._marshal(0)
        self._destroy()


class ZwpTabletToolV2Proxy(Proxy[ZwpTabletToolV2]):
    interface = ZwpTabletToolV2

    @ZwpTabletToolV2.request(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Object, interface=WlSurface, nullable=True),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def set_cursor(self, serial: int, surface: WlSurfaceProxy | None, hotspot_x: int, hotspot_y: int) -> None:
        """Set the tablet tool's surface

        Sets the surface of the cursor used for this tool on the given tablet.
        This request only takes effect if the tool is in proximity of one of
        the requesting client's surfaces or the surface parameter is the
        current pointer surface. If there was a previous surface set with this
        request it is replaced. If surface is NULL, the cursor image is hidden.

        The parameters hotspot_x and hotspot_y define the position of the
        pointer surface relative to the pointer location. Its top-left corner
        is always at (x, y) - (hotspot_x, hotspot_y), where (x, y) are the
        coordinates of the pointer location, in surface-local coordinates.

        On surface.attach requests to the pointer surface, hotspot_x and
        hotspot_y are decremented by the x and y parameters passed to the
        request. Attach must be confirmed by :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>` as usual.

        The hotspot can also be updated by passing the currently set pointer
        surface to this request with new values for hotspot_x and hotspot_y.

        The current and pending input regions of the
        :class:`~pywayland.protocol.wayland.WlSurface` are cleared, and
        :func:`WlSurface.set_input_region()
        <pywayland.protocol.wayland.WlSurface.set_input_region>` is ignored
        until the :class:`~pywayland.protocol.wayland.WlSurface` is no longer
        used as the cursor. When the use as a cursor ends, the current and
        pending input regions become undefined, and the
        :class:`~pywayland.protocol.wayland.WlSurface` is unmapped.

        This request gives the surface the role of a :class:`ZwpTabletToolV2`
        cursor. A surface may only ever be used as the cursor surface for one
        :class:`ZwpTabletToolV2`. If the surface already has another role or
        has previously been used as cursor surface for a different tool, a
        protocol error is raised.

        :param serial:
            serial of the proximity_in event
        :type serial:
            `ArgumentType.Uint`
        :param surface:
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface` or `None`
        :param hotspot_x:
            surface-local x coordinate
        :type hotspot_x:
            `ArgumentType.Int`
        :param hotspot_y:
            surface-local y coordinate
        :type hotspot_y:
            `ArgumentType.Int`
        """
        self._marshal(0, serial, surface, hotspot_x, hotspot_y)

    @ZwpTabletToolV2.request()
    def destroy(self) -> None:
        """Destroy the tool object

        This destroys the client's resource for this tool object.
        """
        self._marshal(1)
        self._destroy()


class ZwpTabletV2Proxy(Proxy[ZwpTabletV2]):
    interface = ZwpTabletV2

    @ZwpTabletV2.request()
    def destroy(self) -> None:
        """Destroy the tablet object

        This destroys the client's resource for this tablet object.
        """
        self._marshal(0)
        self._destroy()


class ZwpTabletManagerV2Global(Global[ZwpTabletManagerV2]):
    interface = ZwpTabletManagerV2


class ZwpTabletPadDialV2Global(Global[ZwpTabletPadDialV2]):
    interface = ZwpTabletPadDialV2


class ZwpTabletPadGroupV2Global(Global[ZwpTabletPadGroupV2]):
    interface = ZwpTabletPadGroupV2


class ZwpTabletPadRingV2Global(Global[ZwpTabletPadRingV2]):
    interface = ZwpTabletPadRingV2


class ZwpTabletPadStripV2Global(Global[ZwpTabletPadStripV2]):
    interface = ZwpTabletPadStripV2


class ZwpTabletPadV2Global(Global[ZwpTabletPadV2]):
    interface = ZwpTabletPadV2


class ZwpTabletSeatV2Global(Global[ZwpTabletSeatV2]):
    interface = ZwpTabletSeatV2


class ZwpTabletToolV2Global(Global[ZwpTabletToolV2]):
    interface = ZwpTabletToolV2


class ZwpTabletV2Global(Global[ZwpTabletV2]):
    interface = ZwpTabletV2


ZwpTabletManagerV2._gen_c()
ZwpTabletManagerV2.proxy_class = ZwpTabletManagerV2Proxy
ZwpTabletManagerV2.resource_class = ZwpTabletManagerV2Resource
ZwpTabletManagerV2.global_class = ZwpTabletManagerV2Global


ZwpTabletPadDialV2._gen_c()
ZwpTabletPadDialV2.proxy_class = ZwpTabletPadDialV2Proxy
ZwpTabletPadDialV2.resource_class = ZwpTabletPadDialV2Resource
ZwpTabletPadDialV2.global_class = ZwpTabletPadDialV2Global


ZwpTabletPadGroupV2._gen_c()
ZwpTabletPadGroupV2.proxy_class = ZwpTabletPadGroupV2Proxy
ZwpTabletPadGroupV2.resource_class = ZwpTabletPadGroupV2Resource
ZwpTabletPadGroupV2.global_class = ZwpTabletPadGroupV2Global


ZwpTabletPadRingV2._gen_c()
ZwpTabletPadRingV2.proxy_class = ZwpTabletPadRingV2Proxy
ZwpTabletPadRingV2.resource_class = ZwpTabletPadRingV2Resource
ZwpTabletPadRingV2.global_class = ZwpTabletPadRingV2Global


ZwpTabletPadStripV2._gen_c()
ZwpTabletPadStripV2.proxy_class = ZwpTabletPadStripV2Proxy
ZwpTabletPadStripV2.resource_class = ZwpTabletPadStripV2Resource
ZwpTabletPadStripV2.global_class = ZwpTabletPadStripV2Global


ZwpTabletPadV2._gen_c()
ZwpTabletPadV2.proxy_class = ZwpTabletPadV2Proxy
ZwpTabletPadV2.resource_class = ZwpTabletPadV2Resource
ZwpTabletPadV2.global_class = ZwpTabletPadV2Global


ZwpTabletSeatV2._gen_c()
ZwpTabletSeatV2.proxy_class = ZwpTabletSeatV2Proxy
ZwpTabletSeatV2.resource_class = ZwpTabletSeatV2Resource
ZwpTabletSeatV2.global_class = ZwpTabletSeatV2Global


ZwpTabletToolV2._gen_c()
ZwpTabletToolV2.proxy_class = ZwpTabletToolV2Proxy
ZwpTabletToolV2.resource_class = ZwpTabletToolV2Resource
ZwpTabletToolV2.global_class = ZwpTabletToolV2Global


ZwpTabletV2._gen_c()
ZwpTabletV2.proxy_class = ZwpTabletV2Proxy
ZwpTabletV2.resource_class = ZwpTabletV2Resource
ZwpTabletV2.global_class = ZwpTabletV2Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2015-2016 Red Hat Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from __future__ import annotations

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource

from .wayland import WlSurface
from .wayland import WlSurfaceProxy


class ZxdgExportedV2(Interface):
    """An exported surface handle

    An xdg_exported object represents an exported reference to a surface. The
    exported surface may be referenced as long as the xdg_exported object not
    destroyed. Destroying the xdg_exported invalidates any relationship the
    importer may have established using xdg_imported.
    """

    name = "zxdg_exported_v2"
    version = 1


class ZxdgExporterV2(Interface):
    """Interface for exporting surfaces

    A global interface used for exporting surfaces that can later be imported
    using xdg_importer.
    """

    name = "zxdg_exporter_v2"
    version = 1

    class error(enum.IntEnum):
        invalid_surface = 0


class ZxdgImportedV2(Interface):
    """An imported surface handle

    An xdg_imported object represents an imported reference to surface exported
    by some client. A client can use this interface to manipulate relationships
    between its own surfaces and the imported surface.
    """

    name = "zxdg_imported_v2"
    version = 1

    class error(enum.IntEnum):
        invalid_surface = 0


class ZxdgImporterV2(Interface):
    """Interface for importing surfaces

    A global interface used for importing surfaces exported by xdg_exporter.
    With this interface, a client can create a reference to a surface of
    another client.
    """

    name = "zxdg_importer_v2"
    version = 1


class ZxdgExportedV2Resource(Resource[ZxdgExportedV2]):
    interface = ZxdgExportedV2

    @ZxdgExportedV2.event(
        Argument(ArgumentType.String),
    )
    def handle(self, handle: str) -> None:
        """The exported surface handle

        The handle event contains the unique handle of this exported surface
        reference. It may be shared with any client, which then can use it to
        import the surface by calling xdg_importer.import_toplevel. A handle
        may be used to import the surface multiple times.

        :param handle:
            the exported surface handle
        :type handle:
            `ArgumentType.String`
        """
        self._post_event(0, handle)


class ZxdgExporterV2Resource(Resource[ZxdgExporterV2]):
    interface = ZxdgExporterV2


class ZxdgImportedV2Resource(Resource[ZxdgImportedV2]):
    interface = ZxdgImportedV2

    @ZxdgImportedV2.event()
    def destroyed(self) -> None:
        """The imported surface handle has been destroyed

        The imported surface handle has been destroyed and any relationship set
        up has been invalidated. This may happen for various reasons, for
        example if the exported surface or the exported surface handle has been
        destroyed, if the handle used for importing was invalid.
        """
        self._post_event(0)


class ZxdgImporterV2Resource(Resource[ZxdgImporterV2]):
    interface = ZxdgImporterV2


class ZxdgExportedV2Proxy(Proxy[ZxdgExportedV2]):
    interface = ZxdgExportedV2

    @ZxdgExportedV2.request()
    def destroy(self) -> None:
        """Unexport the exported surface

        Revoke the previously exported surface. This invalidates any
        relationship the importer may have set up using the xdg_imported
        created given the handle sent via xdg_exported.handle.
        """
        self._marshal(0)
        self._destroy()


class ZxdgExporterV2Proxy(Proxy[ZxdgExporterV2]):
    interface = ZxdgExporterV2

    @ZxdgExporterV2.request()
    def destroy(self) -> None:
        """Destroy the xdg_exporter object

        Notify the compositor that the xdg_exporter object will no longer be
        used.
        """
        self._marshal(0)
        self._destroy()

    @ZxdgExporterV2.request(
        Argument(ArgumentType.NewId, interface=ZxdgExportedV2),
        Argument(ArgumentType.Object, interface=WlSurface),
    )
    def export_toplevel(self, surface: WlSurfaceProxy) -> ZxdgExportedV2Proxy:
        """Export a toplevel surface

        The export_toplevel request exports the passed surface so that it can
        later be imported via xdg_importer. When called, a new xdg_exported
        object will be created and xdg_exported.handle will be sent
        immediately. See the corresponding interface and event for details.

        A surface may be exported multiple times, and each exported handle may
        be used to create an xdg_imported multiple times. Only
        :class:`~pywayland.protocol.xdg_shell.XdgToplevel` equivalent surfaces
        may be exported, otherwise an invalid_surface protocol error is sent.

        :param surface:
            the surface to export
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        :returns:
            :class:`~pywayland.protocol.xdg_foreign_unstable_v2.ZxdgExportedV2`
            -- the new xdg_exported object
        """
        id = self._marshal_constructor(1, ZxdgExportedV2, surface)
        return id


class ZxdgImportedV2Proxy(Proxy[ZxdgImportedV2]):
    interface = ZxdgImportedV2

    @ZxdgImportedV2.request()
    def destroy(self) -> None:
        """Destroy the xdg_imported object

        Notify the compositor that it will no longer use the xdg_imported
        object. Any relationship that may have been set up will at this point
        be invalidated.
        """
        self._marshal(0)
        self._destroy()

    @ZxdgImportedV2.request(
        Argument(ArgumentType.Object, interface=WlSurface),
    )
    def set_parent_of(self, surface: WlSurfaceProxy) -> None:
        """Set as the parent of some surface

        Set the imported surface as the parent of some surface of the client.
        The passed surface must be an
        :class:`~pywayland.protocol.xdg_shell.XdgToplevel` equivalent,
        otherwise an invalid_surface protocol error is sent. Calling this
        function sets up a surface to surface relation with the same stacking
        and positioning semantics as :func:`XdgToplevel.set_parent()
        <pywayland.protocol.xdg_shell.XdgToplevel.set_parent>`.

        :param surface:
            the child surface
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        """
        self._marshal(1, surface)


class ZxdgImporterV2Proxy(Proxy[ZxdgImporterV2]):
    interface = ZxdgImporterV2

    @ZxdgImporterV2.request()
    def destroy(self) -> None:
        """Destroy the xdg_importer object

        Notify the compositor that the xdg_importer object will no longer be
        used.
        """
        self._marshal(0)
        self._destroy()

    @ZxdgImporterV2.request(
        Argument(ArgumentType.NewId, interface=ZxdgImportedV2),
        Argument(ArgumentType.String),
    )
    def import_toplevel(self, handle: str) -> ZxdgImportedV2Proxy:
        """Import a toplevel surface

        The import_toplevel request imports a surface from any client given a
        handle retrieved by exporting said surface using
        xdg_exporter.export_toplevel. When called, a new xdg_imported object
        will be created. This new object represents the imported surface, and
        the importing client can manipulate its relationship using it. See
        xdg_imported for details.

        :param handle:
            the exported surface handle
        :type handle:
            `ArgumentType.String`
        :returns:
            :class:`~pywayland.protocol.xdg_foreign_unstable_v2.ZxdgImportedV2`
            -- the new xdg_imported object
        """
        id = self._marshal_constructor(1, ZxdgImportedV2, handle)
        return id


class ZxdgExportedV2Global(Global[ZxdgExportedV2]):
    interface = ZxdgExportedV2


class ZxdgExporterV2Global(Global[ZxdgExporterV2]):
    interface = ZxdgExporterV2


class ZxdgImportedV2Global(Global[ZxdgImportedV2]):
    interface = ZxdgImportedV2


class ZxdgImporterV2Global(Global[ZxdgImporterV2]):
    interface = ZxdgImporterV2


ZxdgExportedV2._gen_c()
ZxdgExportedV2.proxy_class = ZxdgExportedV2Proxy
ZxdgExportedV2.resource_class = ZxdgExportedV2Resource
ZxdgExportedV2.global_class = ZxdgExportedV2Global


ZxdgExporterV2._gen_c()
ZxdgExporterV2.proxy_class = ZxdgExporterV2Proxy
ZxdgExporterV2.resource_class = ZxdgExporterV2Resource
ZxdgExporterV2.global_class = ZxdgExporterV2Global


ZxdgImportedV2._gen_c()
ZxdgImportedV2.proxy_class = ZxdgImportedV2Proxy
ZxdgImportedV2.resource_class = ZxdgImportedV2Resource
ZxdgImportedV2.global_class = ZxdgImportedV2Global


ZxdgImporterV2._gen_c()
ZxdgImporterV2.proxy_class = ZxdgImporterV2Proxy
ZxdgImporterV2.resource_class = ZxdgImporterV2Resource
ZxdgImporterV2.global_class = ZxdgImporterV2Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2019 Christopher Billington
# Copyright © 2020 Ilia Bozhinov
# Copyright © 2022 Victoria Brekenfeld
#
# Permission to use, copy, modify, distribute, and sell this
# software and its documentation for any purpose is hereby granted
# without fee, provided that the above copyright notice appear in
# all copies and that both that copyright notice and this permission
# notice appear in supporting documentation, and that the name of
# the copyright holders not be used in advertising or publicity
# pertaining to distribution of the software without specific,
# written prior permission.  The copyright holders make no
# representations about the suitability of this software for any
# purpose.  It is provided "as is" without express or implied
# warranty.
#
# THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
# SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
# SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
# AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
# ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF
# THIS SOFTWARE.

from __future__ import annotations

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource

from .wayland import WlOutput


class ExtWorkspaceGroupHandleV1(Interface):
    """A workspace group assigned to a set of outputs

    A :class:`ExtWorkspaceGroupHandleV1` object represents a workspace group
    that is assigned a set of outputs and contains a number of workspaces.

    The set of outputs assigned to the workspace group is conveyed to the
    client via output_enter and output_leave events, and its workspaces are
    conveyed with workspace events.

    For example, a compositor which has a set of workspaces for each output may
    advertise a workspace group (and its workspaces) per output, whereas a
    compositor where a workspace spans all outputs may advertise a single
    workspace group for all outputs.
    """

    name = "ext_workspace_group_handle_v1"
    version = 1

    class group_capabilities(enum.IntFlag):
        create_workspace = 1


class ExtWorkspaceHandleV1(Interface):
    """A workspace handing a group of surfaces

    A :class:`ExtWorkspaceHandleV1` object represents a workspace that handles
    a group of surfaces.

    Each workspace has: - a name, conveyed to the client with the name event -
    potentially an id conveyed with the id event - a list of states, conveyed
    to the client with the state event - and optionally a set of coordinates,
    conveyed to the client with the coordinates event

    The client may request that the compositor activate or deactivate the
    workspace.

    Each workspace can belong to only a single workspace group. Depending on
    the compositor policy, there might be workspaces with the same name in
    different workspace groups, but these workspaces are still separate (e.g.
    one of them might be active while the other is not).
    """

    name = "ext_workspace_handle_v1"
    version = 1

    class state(enum.IntFlag):
        active = 1
        urgent = 2
        hidden = 4

    class workspace_capabilities(enum.IntFlag):
        activate = 1
        deactivate = 2
        remove = 4
        assign = 8


class ExtWorkspaceManagerV1(Interface):
    """List and control workspaces

    Workspaces, also called virtual desktops, are groups of surfaces. A
    compositor with a concept of workspaces may only show some such groups of
    surfaces (those of 'active' workspaces) at a time. 'Activating' a workspace
    is a request for the compositor to display that workspace's surfaces as
    normal, whereas the compositor may hide or otherwise de-emphasise surfaces
    that are associated only with 'inactive' workspaces. Workspaces are grouped
    by which sets of outputs they correspond to, and may contain surfaces only
    from those outputs. In this way, it is possible for each output to have its
    own set of workspaces, or for all outputs (or any other arbitrary grouping)
    to share workspaces. Compositors may optionally conceptually arrange each
    group of workspaces in an N-dimensional grid.

    The purpose of this protocol is to enable the creation of taskbars and
    docks by providing them with a list of workspaces and their properties, and
    allowing them to activate and deactivate workspaces.

    After a client binds the :class:`ExtWorkspaceManagerV1`, each workspace
    will be sent via the workspace event.
    """

    name = "ext_workspace_manager_v1"
    version = 1


class ExtWorkspaceGroupHandleV1Resource(Resource[ExtWorkspaceGroupHandleV1]):
    interface = ExtWorkspaceGroupHandleV1

    @ExtWorkspaceGroupHandleV1.event(
        Argument(ArgumentType.Uint),
    )
    def capabilities(self, capabilities: int) -> None:
        """Compositor capabilities

        This event advertises the capabilities supported by the compositor. If
        a capability isn't supported, clients should hide or disable the UI
        elements that expose this functionality. For instance, if the
        compositor doesn't advertise support for creating workspaces, a button
        triggering the create_workspace request should not be displayed.

        The compositor will ignore requests it doesn't support. For instance, a
        compositor which doesn't advertise support for creating workspaces will
        ignore create_workspace requests.

        Compositors must send this event once after creation of an
        :class:`ExtWorkspaceGroupHandleV1`. When the capabilities change,
        compositors must send this event again.

        :param capabilities:
            capabilities
        :type capabilities:
            `ArgumentType.Uint`
        """
        self._post_event(0, capabilities)

    @ExtWorkspaceGroupHandleV1.event(
        Argument(ArgumentType.Object, interface=WlOutput),
    )
    def output_enter(self, output: WlOutput) -> None:
        """Output assigned to workspace group

        This event is emitted whenever an output is assigned to the workspace
        group or a new `:class:`~pywayland.protocol.wayland.WlOutput`` object
        is bound by the client, which was already assigned to this
        workspace_group.

        :param output:
        :type output:
            :class:`~pywayland.protocol.wayland.WlOutput`
        """
        self._post_event(1, output)

    @ExtWorkspaceGroupHandleV1.event(
        Argument(ArgumentType.Object, interface=WlOutput),
    )
    def output_leave(self, output: WlOutput) -> None:
        """Output removed from workspace group

        This event is emitted whenever an output is removed from the workspace
        group.

        :param output:
        :type output:
            :class:`~pywayland.protocol.wayland.WlOutput`
        """
        self._post_event(2, output)

    @ExtWorkspaceGroupHandleV1.event(
        Argument(ArgumentType.Object, interface=ExtWorkspaceHandleV1),
    )
    def workspace_enter(self, workspace: ExtWorkspaceHandleV1) -> None:
        """Workspace added to workspace group

        This event is emitted whenever a workspace is assigned to this group. A
        workspace may only ever be assigned to a single group at a single point
        in time, but can be re-assigned during its lifetime.

        :param workspace:
        :type workspace:
            :class:`~pywayland.protocol.ext_workspace_v1.ExtWorkspaceHandleV1`
        """
        self._post_event(3, workspace)

    @ExtWorkspaceGroupHandleV1.event(
        Argument(ArgumentType.Object, interface=ExtWorkspaceHandleV1),
    )
    def workspace_leave(self, workspace: ExtWorkspaceHandleV1) -> None:
        """Workspace removed from workspace group

        This event is emitted whenever a workspace is removed from this group.

        :param workspace:
        :type workspace:
            :class:`~pywayland.protocol.ext_workspace_v1.ExtWorkspaceHandleV1`
        """
        self._post_event(4, workspace)

    @ExtWorkspaceGroupHandleV1.event()
    def removed(self) -> None:
        """This workspace group has been removed

        This event is send when the group associated with the
        :class:`ExtWorkspaceGroupHandleV1` has been removed. After sending this
        request the compositor will immediately consider the object inert. Any
        requests will be ignored except the destroy request. It is guaranteed
        there won't be any more events referencing this
        :class:`ExtWorkspaceGroupHandleV1`.

        The compositor must remove all workspaces belonging to a workspace
        group via a workspace_leave event before removing the workspace group.
        """
        self._post_event(5)


class ExtWorkspaceHandleV1Resource(Resource[ExtWorkspaceHandleV1]):
    interface = ExtWorkspaceHandleV1

    @ExtWorkspaceHandleV1.event(
        Argument(ArgumentType.String),
    )
    def id(self, id: str) -> None:
        """Workspace id

        If this event is emitted, it will be send immediately after the
        :class:`ExtWorkspaceHandleV1` is created or when an id is assigned to a
        workspace (at most once during its lifetime).

        An id will never change during the lifetime of the
        `:class:`ExtWorkspaceHandleV1`` and is guaranteed to be unique during
        its lifetime.

        Ids are not human-readable and shouldn't be displayed, use `name` for
        that purpose.

        Compositors are expected to only send ids for workspaces likely stable
        across multiple sessions and can be used by clients to store
        preferences for workspaces. Workspaces without ids should be considered
        temporary and any data associated with them should be deleted once the
        respective object is lost.

        :param id:
        :type id:
            `ArgumentType.String`
        """
        self._post_event(0, id)

    @ExtWorkspaceHandleV1.event(
        Argument(ArgumentType.String),
    )
    def name(self, name: str) -> None:
        """Workspace name changed

        This event is emitted immediately after the
        :class:`ExtWorkspaceHandleV1` is created and whenever the name of the
        workspace changes.

        A name is meant to be human-readable and can be displayed to a user.
        Unlike the id it is neither stable nor unique.

        :param name:
        :type name:
            `ArgumentType.String`
        """
        self._post_event(1, name)

    @ExtWorkspaceHandleV1.event(
        Argument(ArgumentType.Array),
    )
    def coordinates(self, coordinates: list) -> None:
        """Workspace coordinates changed

        This event is used to organize workspaces into an N-dimensional grid
        within a workspace group, and if supported, is emitted immediately
        after the :class:`ExtWorkspaceHandleV1` is created and whenever the
        coordinates of the workspace change. Compositors may not send this
        event if they do not conceptually arrange workspaces in this way. If
        compositors simply number workspaces, without any geometric
        interpretation, they may send 1D coordinates, which clients should not
        interpret as implying any geometry. Sending an empty array means that
        the compositor no longer orders the workspace geometrically.

        Coordinates have an arbitrary number of dimensions N with an uint32
        position along each dimension. By convention if N > 1, the first
        dimension is X, the second Y, the third Z, and so on. The compositor
        may chose to utilize these events for a more novel workspace layout
        convention, however. No guarantee is made about the grid being filled
        or bounded; there may be a workspace at coordinate 1 and another at
        coordinate 1000 and none in between. Within a workspace group, however,
        workspaces must have unique coordinates of equal dimensionality.

        :param coordinates:
        :type coordinates:
            `ArgumentType.Array`
        """
        self._post_event(2, coordinates)

    @ExtWorkspaceHandleV1.event(
        Argument(ArgumentType.Uint),
    )
    def state(self, state: int) -> None:
        """The state of the workspace changed

        This event is emitted immediately after the
        :class:`ExtWorkspaceHandleV1` is created and each time the workspace
        state changes, either because of a compositor action or because of a
        request in this protocol.

        Missing states convey the opposite meaning, e.g. an unset active bit
        means the workspace is currently inactive.

        :param state:
        :type state:
            `ArgumentType.Uint`
        """
        self._post_event(3, state)

    @ExtWorkspaceHandleV1.event(
        Argument(ArgumentType.Uint),
    )
    def capabilities(self, capabilities: int) -> None:
        """Compositor capabilities

        This event advertises the capabilities supported by the compositor. If
        a capability isn't supported, clients should hide or disable the UI
        elements that expose this functionality. For instance, if the
        compositor doesn't advertise support for removing workspaces, a button
        triggering the remove request should not be displayed.

        The compositor will ignore requests it doesn't support. For instance, a
        compositor which doesn't advertise support for remove will ignore
        remove requests.

        Compositors must send this event once after creation of an
        :class:`ExtWorkspaceHandleV1` . When the capabilities change,
        compositors must send this event again.

        :param capabilities:
            capabilities
        :type capabilities:
            `ArgumentType.Uint`
        """
        self._post_event(4, capabilities)

    @ExtWorkspaceHandleV1.event()
    def removed(self) -> None:
        """This workspace has been removed

        This event is send when the workspace associated with the
        :class:`ExtWorkspaceHandleV1` has been removed. After sending this
        request, the compositor will immediately consider the object inert. Any
        requests will be ignored except the destroy request.

        It is guaranteed there won't be any more events referencing this
        :class:`ExtWorkspaceHandleV1`.

        The compositor must only remove a workspaces not currently belonging to
        any workspace_group.
        """
        self._post_event(5)


class ExtWorkspaceManagerV1Resource(Resource[ExtWorkspaceManagerV1]):
    interface = ExtWorkspaceManagerV1

    @ExtWorkspaceManagerV1.event(
        Argument(ArgumentType.NewId, interface=ExtWorkspaceGroupHandleV1),
    )
    def workspace_group(self, workspace_group: ExtWorkspaceGroupHandleV1) -> None:
        """A workspace group has been created

        This event is emitted whenever a new workspace group has been created.

        All initial details of the workspace group (outputs) will be sent
        immediately after this event via the corresponding events in
        :class:`~pywayland.protocol.ext_workspace_v1.ExtWorkspaceGroupHandleV1`
        and :class:`~pywayland.protocol.ext_workspace_v1.ExtWorkspaceHandleV1`.

        :param workspace_group:
        :type workspace_group:
            :class:`~pywayland.protocol.ext_workspace_v1.ExtWorkspaceGroupHandleV1`
        """
        self._post_event(0, workspace_group)

    @ExtWorkspaceManagerV1.event(
        Argument(ArgumentType.NewId, interface=ExtWorkspaceHandleV1),
    )
    def workspace(self, workspace: ExtWorkspaceHandleV1) -> None:
        """Workspace has been created

        This event is emitted whenever a new workspace has been created.

        All initial details of the workspace (name, coordinates, state) will be
        sent immediately after this event via the corresponding events in
        :class:`~pywayland.protocol.ext_workspace_v1.ExtWorkspaceHandleV1`.

        Workspaces start off unassigned to any workspace group.

        :param workspace:
        :type workspace:
            :class:`~pywayland.protocol.ext_workspace_v1.ExtWorkspaceHandleV1`
        """
        self._post_event(1, workspace)

    @ExtWorkspaceManagerV1.event()
    def done(self) -> None:
        """All information about the workspaces and workspace groups has been sent

        This event is sent after all changes in all workspaces and workspace
        groups have been sent.

        This allows changes to one or more
        :class:`~pywayland.protocol.ext_workspace_v1.ExtWorkspaceGroupHandleV1`
        properties and
        :class:`~pywayland.protocol.ext_workspace_v1.ExtWorkspaceHandleV1`
        properties to be seen as atomic, even if they happen via multiple
        events. In particular, an output moving from one workspace group to
        another sends an output_enter event and an output_leave event to the
        two
        :class:`~pywayland.protocol.ext_workspace_v1.ExtWorkspaceGroupHandleV1`
        objects in question. The compositor sends the done event only after
        updating the output information in both workspace groups.
        """
        self._post_event(2)

    @ExtWorkspaceManagerV1.event()
    def finished(self) -> None:
        """The compositor has finished with the workspace_manager

        This event indicates that the compositor is done sending events to the
        :class:`ExtWorkspaceManagerV1`. The server will destroy the object
        immediately after sending this request.
        """
        self._post_event(3)


class ExtWorkspaceGroupHandleV1Proxy(Proxy[ExtWorkspaceGroupHandleV1]):
    interface = ExtWorkspaceGroupHandleV1

    @ExtWorkspaceGroupHandleV1.request(
        Argument(ArgumentType.String),
    )
    def create_workspace(self, workspace: str) -> None:
        """Create a new workspace

        Request that the compositor create a new workspace with the given name
        and assign it to this group.

        There is no guarantee that the compositor will create a new workspace,
        or that the created workspace will have the provided name.

        :param workspace:
        :type workspace:
            `ArgumentType.String`
        """
        self._marshal(0, workspace)

    @ExtWorkspaceGroupHandleV1.request()
    def destroy(self) -> None:
        """Destroy the :class:`ExtWorkspaceGroupHandleV1` object

        Destroys the :class:`ExtWorkspaceGroupHandleV1` object.

        This request should be send either when the client does not want to use
        the workspace group object any more or after the removed event to
        finalize the destruction of the object.
        """
        self._marshal(1)
        self._destroy()


class ExtWorkspaceHandleV1Proxy(Proxy[ExtWorkspaceHandleV1]):
    interface = ExtWorkspaceHandleV1

    @ExtWorkspaceHandleV1.request()
    def destroy(self) -> None:
        """Destroy the :class:`ExtWorkspaceHandleV1` object

        Destroys the :class:`ExtWorkspaceHandleV1` object.

        This request should be made either when the client does not want to use
        the workspace object any more or after the remove event to finalize the
        destruction of the object.
        """
        self._marshal(0)
        self._destroy()

    @ExtWorkspaceHandleV1.request()
    def activate(self) -> None:
        """Activate the workspace

        Request that this workspace be activated.

        There is no guarantee the workspace will be actually activated, and
        behaviour may be compositor-dependent. For example, activating a
        workspace may or may not deactivate all other workspaces in the same
        group.
        """
        self._marshal(1)

    @ExtWorkspaceHandleV1.request()
    def deactivate(self) -> None:
        """Deactivate the workspace

        Request that this workspace be deactivated.

        There is no guarantee the workspace will be actually deactivated.
        """
        self._marshal(2)

    @ExtWorkspaceHandleV1.request(
        Argument(ArgumentType.Object, interface=ExtWorkspaceGroupHandleV1),
    )
    def assign(self, workspace_group: ExtWorkspaceGroupHandleV1Proxy) -> None:
        """Assign workspace to group

        Requests that this workspace is assigned to the given workspace group.

        There is no guarantee the workspace will be assigned.

        :param workspace_group:
        :type workspace_group:
            :class:`~pywayland.protocol.ext_workspace_v1.ExtWorkspaceGroupHandleV1`
        """
        self._marshal(3, workspace_group)

    @ExtWorkspaceHandleV1.request()
    def remove(self) -> None:
        """Remove the workspace

        Request that this workspace be removed.

        There is no guarantee the workspace will be actually removed.
        """
        self._marshal(4)


class ExtWorkspaceManagerV1Proxy(Proxy[ExtWorkspaceManagerV1]):
    interface = ExtWorkspaceManagerV1

    @ExtWorkspaceManagerV1.request()
    def commit(self) -> None:
        """All requests about the workspaces have been sent

        The client must send this request after it has finished sending other
        requests. The compositor must process a series of requests preceding a
        commit request atomically.

        This allows changes to the workspace properties to be seen as atomic,
        even if they happen via multiple events, and even if they involve
        multiple
        :class:`~pywayland.protocol.ext_workspace_v1.ExtWorkspaceHandleV1`
        objects, for example, deactivating one workspace and activating
        another.
        """
        self._marshal(0)

    @ExtWorkspaceManagerV1.request()
    def stop(self) -> None:
        """Stop sending events

        Indicates the client no longer wishes to receive events for new
        workspace groups. However the compositor may emit further workspace
        events, until the finished event is emitted. The compositor is expected
        to send the finished event eventually once the stop request has been
        processed.

        The client must not send any requests after this one, doing so will
        raise a :class:`~pywayland.protocol.wayland.WlDisplay` invalid_object
        error.
        """
        self._marshal(1)


class ExtWorkspaceGroupHandleV1Global(Global[ExtWorkspaceGroupHandleV1]):
    interface = ExtWorkspaceGroupHandleV1


class ExtWorkspaceHandleV1Global(Global[ExtWorkspaceHandleV1]):
    interface = ExtWorkspaceHandleV1


class ExtWorkspaceManagerV1Global(Global[ExtWorkspaceManagerV1]):
    interface = ExtWorkspaceManagerV1


ExtWorkspaceGroupHandleV1._gen_c()
ExtWorkspaceGroupHandleV1.proxy_class = ExtWorkspaceGroupHandleV1Proxy
ExtWorkspaceGroupHandleV1.resource_class = ExtWorkspaceGroupHandleV1Resource
ExtWorkspaceGroupHandleV1.global_class = ExtWorkspaceGroupHandleV1Global


ExtWorkspaceHandleV1._gen_c()
ExtWorkspaceHandleV1.proxy_class = ExtWorkspaceHandleV1Proxy
ExtWorkspaceHandleV1.resource_class = ExtWorkspaceHandleV1Resource
ExtWorkspaceHandleV1.global_class = ExtWorkspaceHandleV1Global


ExtWorkspaceManagerV1._gen_c()
ExtWorkspaceManagerV1.proxy_class = ExtWorkspaceManagerV1Proxy
ExtWorkspaceManagerV1.resource_class = ExtWorkspaceManagerV1Resource
ExtWorkspaceManagerV1.global_class = ExtWorkspaceManagerV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2023 Valve Corporation
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from __future__ import annotations

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource

from .wayland import WlSurface
from .wayland import WlSurfaceProxy


class WpCommitTimerV1(Interface):
    """Surface commit timer

    An object to set a time constraint for a content update on a surface.
    """

    name = "wp_commit_timer_v1"
    version = 1

    class error(enum.IntEnum):
        invalid_timestamp = 0
        timestamp_exists = 1
        surface_destroyed = 2


class WpCommitTimingManagerV1(Interface):
    """Commit timing

    When a compositor latches on to new content updates it will check for any
    number of requirements of the available content updates (such as fences of
    all buffers being signalled) to consider the update ready.

    This protocol provides a method for adding a time constraint to surface
    content. This constraint indicates to the compositor that a content update
    should be presented as closely as possible to, but not before, a specified
    time.

    This protocol does not change the Wayland property that content updates are
    applied in the order they are received, even when some content updates
    contain timestamps and others do not.

    To provide timestamps, this global factory interface must be used to
    acquire a wp_commit_timing_v1 object for a surface, which may then be used
    to provide timestamp information for commits.

    Warning! The protocol described in this file is currently in the testing
    phase. Backward compatible changes may be added together with the
    corresponding interface version bump. Backward incompatible changes can
    only be done by creating a new major version of the extension.
    """

    name = "wp_commit_timing_manager_v1"
    version = 1

    class error(enum.IntEnum):
        commit_timer_exists = 0


class WpCommitTimerV1Resource(Resource[WpCommitTimerV1]):
    interface = WpCommitTimerV1


class WpCommitTimingManagerV1Resource(Resource[WpCommitTimingManagerV1]):
    interface = WpCommitTimingManagerV1


class WpCommitTimerV1Proxy(Proxy[WpCommitTimerV1]):
    interface = WpCommitTimerV1

    @WpCommitTimerV1.request(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def set_timestamp(self, tv_sec_hi: int, tv_sec_lo: int, tv_nsec: int) -> None:
        """Specify time the following commit takes effect

        Provide a timing constraint for a surface content update.

        A set_timestamp request may be made before a :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>` to tell the compositor
        that the content is intended to be presented as closely as possible to,
        but not before, the specified time. The time is in the domain of the
        compositor's presentation clock.

        An invalid_timestamp error will be generated for invalid tv_nsec.

        If a timestamp already exists on the surface, a timestamp_exists error
        is generated.

        Requesting set_timestamp after the commit_timer object's surface is
        destroyed will generate a "surface_destroyed" error.

        :param tv_sec_hi:
            high 32 bits of the seconds part of target time
        :type tv_sec_hi:
            `ArgumentType.Uint`
        :param tv_sec_lo:
            low 32 bits of the seconds part of target time
        :type tv_sec_lo:
            `ArgumentType.Uint`
        :param tv_nsec:
            nanoseconds part of target time
        :type tv_nsec:
            `ArgumentType.Uint`
        """
        self._marshal(0, tv_sec_hi, tv_sec_lo, tv_nsec)

    @WpCommitTimerV1.request()
    def destroy(self) -> None:
        """Destroy the timer

        Informs the server that the client will no longer be using this
        protocol object.

        Existing timing constraints are not affected by the destruction.
        """
        self._marshal(1)
        self._destroy()


class WpCommitTimingManagerV1Proxy(Proxy[WpCommitTimingManagerV1]):
    interface = WpCommitTimingManagerV1

    @WpCommitTimingManagerV1.request()
    def destroy(self) -> None:
        """Unbind from the commit timing interface

        Informs the server that the client will no longer be using this
        protocol object. Existing objects created by this object are not
        affected.
        """
        self._marshal(0)
        self._destroy()

    @WpCommitTimingManagerV1.request(
        Argument(ArgumentType.NewId, interface=WpCommitTimerV1),
        Argument(ArgumentType.Object, interface=WlSurface),
    )
    def get_timer(self, surface: WlSurfaceProxy) -> WpCommitTimerV1Proxy:
        """Request commit timer interface for surface

        Establish a timing controller for a surface.

        Only one commit timer can be created for a surface, or a
        commit_timer_exists protocol error will be generated.

        :param surface:
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        :returns:
            :class:`~pywayland.protocol.commit_timing_v1.WpCommitTimerV1`
        """
        id = self._marshal_constructor(1, WpCommitTimerV1, surface)
        return id


class WpCommitTimerV1Global(Global[WpCommitTimerV1]):
    interface = WpCommitTimerV1


class WpCommitTimingManagerV1Global(Global[WpCommitTimingManagerV1]):
    interface = WpCommitTimingManagerV1


WpCommitTimerV1._gen_c()
WpCommitTimerV1.proxy_class = WpCommitTimerV1Proxy
WpCommitTimerV1.resource_class = WpCommitTimerV1Resource
WpCommitTimerV1.global_class = WpCommitTimerV1Global


WpCommitTimingManagerV1._gen_c()
WpCommitTimingManagerV1.proxy_class = WpCommitTimingManagerV1Proxy
WpCommitTimingManagerV1.resource_class = WpCommitTimingManagerV1Resource
WpCommitTimingManagerV1.global_class = WpCommitTimingManagerV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2015-2016 Red Hat Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from __future__ import annotations

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource

from .wayland import WlSurface
from .wayland import WlSurfaceProxy


class ZxdgExportedV1(Interface):
    """An exported surface handle

    An xdg_exported object represents an exported reference to a surface. The
    exported surface may be referenced as long as the xdg_exported object not
    destroyed. Destroying the xdg_exported invalidates any relationship the
    importer may have established using xdg_imported.
    """

    name = "zxdg_exported_v1"
    version = 1


class ZxdgExporterV1(Interface):
    """Interface for exporting surfaces

    A global interface used for exporting surfaces that can later be imported
    using xdg_importer.
    """

    name = "zxdg_exporter_v1"
    version = 1


class ZxdgImportedV1(Interface):
    """An imported surface handle

    An xdg_imported object represents an imported reference to surface exported
    by some client. A client can use this interface to manipulate relationships
    between its own surfaces and the imported surface.
    """

    name = "zxdg_imported_v1"
    version = 1


class ZxdgImporterV1(Interface):
    """Interface for importing surfaces

    A global interface used for importing surfaces exported by xdg_exporter.
    With this interface, a client can create a reference to a surface of
    another client.
    """

    name = "zxdg_importer_v1"
    version = 1


class ZxdgExportedV1Resource(Resource[ZxdgExportedV1]):
    interface = ZxdgExportedV1

    @ZxdgExportedV1.event(
        Argument(ArgumentType.String),
    )
    def handle(self, handle: str) -> None:
        """The exported surface handle

        The handle event contains the unique handle of this exported surface
        reference. It may be shared with any client, which then can use it to
        import the surface by calling xdg_importer.import. A handle may be used
        to import the surface multiple times.

        :param handle:
            the exported surface handle
        :type handle:
            `ArgumentType.String`
        """
        self._post_event(0, handle)


class ZxdgExporterV1Resource(Resource[ZxdgExporterV1]):
    interface = ZxdgExporterV1


class ZxdgImportedV1Resource(Resource[ZxdgImportedV1]):
    interface = ZxdgImportedV1

    @ZxdgImportedV1.event()
    def destroyed(self) -> None:
        """The imported surface handle has been destroyed

        The imported surface handle has been destroyed and any relationship set
        up has been invalidated. This may happen for various reasons, for
        example if the exported surface or the exported surface handle has been
        destroyed, if the handle used for importing was invalid.
        """
        self._post_event(0)


class ZxdgImporterV1Resource(Resource[ZxdgImporterV1]):
    interface = ZxdgImporterV1


class ZxdgExportedV1Proxy(Proxy[ZxdgExportedV1]):
    interface = ZxdgExportedV1

    @ZxdgExportedV1.request()
    def destroy(self) -> None:
        """Unexport the exported surface

        Revoke the previously exported surface. This invalidates any
        relationship the importer may have set up using the xdg_imported
        created given the handle sent via xdg_exported.handle.
        """
        self._marshal(0)
        self._destroy()


class ZxdgExporterV1Proxy(Proxy[ZxdgExporterV1]):
    interface = ZxdgExporterV1

    @ZxdgExporterV1.request()
    def destroy(self) -> None:
        """Destroy the xdg_exporter object

        Notify the compositor that the xdg_exporter object will no longer be
        used.
        """
        self._marshal(0)
        self._destroy()

    @ZxdgExporterV1.request(
        Argument(ArgumentType.NewId, interface=ZxdgExportedV1),
        Argument(ArgumentType.Object, interface=WlSurface),
    )
    def export(self, surface: WlSurfaceProxy) -> ZxdgExportedV1Proxy:
        """Export a surface

        The export request exports the passed surface so that it can later be
        imported via xdg_importer. When called, a new xdg_exported object will
        be created and xdg_exported.handle will be sent immediately. See the
        corresponding interface and event for details.

        A surface may be exported multiple times, and each exported handle may
        be used to create an xdg_imported multiple times. Only
        :class:`~pywayland.protocol.xdg_shell.XdgSurface` surfaces may be
        exported.

        :param surface:
            the surface to export
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        :returns:
            :class:`~pywayland.protocol.xdg_foreign_unstable_v1.ZxdgExportedV1`
            -- the new xdg_exported object
        """
        id = self._marshal_constructor(1, ZxdgExportedV1, surface)
        return id


class ZxdgImportedV1Proxy(Proxy[ZxdgImportedV1]):
    interface = ZxdgImportedV1

    @ZxdgImportedV1.request()
    def destroy(self) -> None:
        """Destroy the xdg_imported object

        Notify the compositor that it will no longer use the xdg_imported
        object. Any relationship that may have been set up will at this point
        be invalidated.
        """
        self._marshal(0)
        self._destroy()

    @ZxdgImportedV1.request(
        Argument(ArgumentType.Object, interface=WlSurface),
    )
    def set_parent_of(self, surface: WlSurfaceProxy) -> None:
        """Set as the parent of some surface

        Set the imported surface as the parent of some surface of the client.
        The passed surface must be a toplevel
        :class:`~pywayland.protocol.xdg_shell.XdgSurface`. Calling this
        function sets up a surface to surface relation with the same stacking
        and positioning semantics as :func:`XdgSurface.set_parent()
        <pywayland.protocol.xdg_shell.XdgSurface.set_parent>`.

        :param surface:
            the child surface
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        """
        self._marshal(1, surface)


class ZxdgImporterV1Proxy(Proxy[ZxdgImporterV1]):
    interface = ZxdgImporterV1

    @ZxdgImporterV1.request()
    def destroy(self) -> None:
        """Destroy the xdg_importer object

        Notify the compositor that the xdg_importer object will no longer be
        used.
        """
        self._marshal(0)
        self._destroy()

    @ZxdgImporterV1.request(
        Argument(ArgumentType.NewId, interface=ZxdgImportedV1),
        Argument(ArgumentType.String),
    )
    def import_(self, handle: str) -> ZxdgImportedV1Proxy:
        """Import a surface

        The import request imports a surface from any client given a handle
        retrieved by exporting said surface using xdg_exporter.export. When
        called, a new xdg_imported object will be created. This new object
        represents the imported surface, and the importing client can
        manipulate its relationship using it. See xdg_imported for details.

        :param handle:
            the exported surface handle
        :type handle:
            `ArgumentType.String`
        :returns:
            :class:`~pywayland.protocol.xdg_foreign_unstable_v1.ZxdgImportedV1`
            -- the new xdg_imported object
        """
        id = self._marshal_constructor(1, ZxdgImportedV1, handle)
        return id


class ZxdgExportedV1Global(Global[ZxdgExportedV1]):
    interface = ZxdgExportedV1


class ZxdgExporterV1Global(Global[ZxdgExporterV1]):
    interface = ZxdgExporterV1


class ZxdgImportedV1Global(Global[ZxdgImportedV1]):
    interface = ZxdgImportedV1


class ZxdgImporterV1Global(Global[ZxdgImporterV1]):
    interface = ZxdgImporterV1


ZxdgExportedV1._gen_c()
ZxdgExportedV1.proxy_class = ZxdgExportedV1Proxy
ZxdgExportedV1.resource_class = ZxdgExportedV1Resource
ZxdgExportedV1.global_class = ZxdgExportedV1Global


ZxdgExporterV1._gen_c()
ZxdgExporterV1.proxy_class = ZxdgExporterV1Proxy
ZxdgExporterV1.resource_class = ZxdgExporterV1Resource
ZxdgExporterV1.global_class = ZxdgExporterV1Global


ZxdgImportedV1._gen_c()
ZxdgImportedV1.proxy_class = ZxdgImportedV1Proxy
ZxdgImportedV1.resource_class = ZxdgImportedV1Resource
ZxdgImportedV1.global_class = ZxdgImportedV1Global


ZxdgImporterV1._gen_c()
ZxdgImporterV1.proxy_class = ZxdgImporterV1Proxy
ZxdgImporterV1.resource_class = ZxdgImporterV1Resource
ZxdgImporterV1.global_class = ZxdgImporterV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2018 Simon Ser
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from __future__ import annotations

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource

from .xdg_shell import XdgToplevel
from .xdg_shell import XdgToplevelProxy


class ZxdgDecorationManagerV1(Interface):
    """Window decoration manager

    This interface allows a compositor to announce support for server-side
    decorations.

    A window decoration is a set of window controls as deemed appropriate by
    the party managing them, such as user interface components used to move,
    resize and change a window's state.

    A client can use this protocol to request being decorated by a supporting
    compositor.

    If compositor and client do not negotiate the use of a server-side
    decoration using this protocol, clients continue to self-decorate as they
    see fit.

    Warning! The protocol described in this file is experimental and backward
    incompatible changes may be made. Backward compatible changes may be added
    together with the corresponding interface version bump. Backward
    incompatible changes are done by bumping the version number in the protocol
    and interface names and resetting the interface version. Once the protocol
    is to be declared stable, the 'z' prefix and the version number in the
    protocol and interface names are removed and the interface version number
    is reset.
    """

    name = "zxdg_decoration_manager_v1"
    version = 2


class ZxdgToplevelDecorationV1(Interface):
    """Decoration object for a toplevel surface

    The decoration object allows the compositor to toggle server-side window
    decorations for a toplevel surface. The client can request to switch to
    another mode.

    The xdg_toplevel_decoration object must be destroyed before its
    :class:`~pywayland.protocol.xdg_shell.XdgToplevel`.
    """

    name = "zxdg_toplevel_decoration_v1"
    version = 2

    class error(enum.IntEnum):
        unconfigured_buffer = 0
        already_constructed = 1
        orphaned = 2
        invalid_mode = 3

    class mode(enum.IntEnum):
        client_side = 1
        server_side = 2


class ZxdgDecorationManagerV1Resource(Resource[ZxdgDecorationManagerV1]):
    interface = ZxdgDecorationManagerV1


class ZxdgToplevelDecorationV1Resource(Resource[ZxdgToplevelDecorationV1]):
    interface = ZxdgToplevelDecorationV1

    @ZxdgToplevelDecorationV1.event(
        Argument(ArgumentType.Uint),
    )
    def configure(self, mode: int) -> None:
        """Notify a decoration mode change

        The configure event configures the effective decoration mode. The
        configured state should not be applied immediately. Clients must send
        an ack_configure in response to this event. See
        :func:`XdgSurface.configure()
        <pywayland.protocol.xdg_shell.XdgSurface.configure>` and
        :func:`XdgSurface.ack_configure()
        <pywayland.protocol.xdg_shell.XdgSurface.ack_configure>` for details.

        A configure event can be sent at any time. The specified mode must be
        obeyed by the client.

        :param mode:
            the decoration mode
        :type mode:
            `ArgumentType.Uint`
        """
        self._post_event(0, mode)


class ZxdgDecorationManagerV1Proxy(Proxy[ZxdgDecorationManagerV1]):
    interface = ZxdgDecorationManagerV1

    @ZxdgDecorationManagerV1.request()
    def destroy(self) -> None:
        """Destroy the decoration manager object

        Destroy the decoration manager. This doesn't destroy objects created
        with the manager.
        """
        self._marshal(0)
        self._destroy()

    @ZxdgDecorationManagerV1.request(
        Argument(ArgumentType.NewId, interface=ZxdgToplevelDecorationV1),
        Argument(ArgumentType.Object, interface=XdgToplevel),
    )
    def get_toplevel_decoration(self, toplevel: XdgToplevelProxy) -> ZxdgToplevelDecorationV1Proxy:
        """Create a new toplevel decoration object

        Create a new decoration object associated with the given toplevel.

        For objects of version 1, creating an xdg_toplevel_decoration from an
        :class:`~pywayland.protocol.xdg_shell.XdgToplevel` which has a buffer
        attached or committed is a client error, and any attempts by a client
        to attach or manipulate a buffer prior to the first
        xdg_toplevel_decoration.configure event must also be treated as errors.

        For objects of version 2 or newer, creating an xdg_toplevel_decoration
        from an :class:`~pywayland.protocol.xdg_shell.XdgToplevel` which has a
        buffer attached or committed is allowed. The initial decoration mode of
        the surface if a buffer is already attached depends on whether a
        xdg_toplevel_decoration object has been associated with the surface or
        not prior to this request.

        If an xdg_toplevel_decoration was associated with the surface, then
        destroyed without a surface commit, the previous decoration mode is
        retained.

        If no xdg_toplevel_decoration was associated with the surface prior to
        this request, or if a surface commit has been performed after a
        previous xdg_toplevel_decoration object associated with the surface was
        destroyed, the decoration mode is assumed to be client-side.

        :param toplevel:
        :type toplevel:
            :class:`~pywayland.protocol.xdg_shell.XdgToplevel`
        :returns:
            :class:`~pywayland.protocol.xdg_decoration_unstable_v1.ZxdgToplevelDecorationV1`
        """
        id = self._marshal_constructor(1, ZxdgToplevelDecorationV1, toplevel)
        return id


class ZxdgToplevelDecorationV1Proxy(Proxy[ZxdgToplevelDecorationV1]):
    interface = ZxdgToplevelDecorationV1

    @ZxdgToplevelDecorationV1.request()
    def destroy(self) -> None:
        """Destroy the decoration object

        Switch back to a mode without any server-side decorations at the next
        commit, unless a new xdg_toplevel_decoration is created for the surface
        first.
        """
        self._marshal(0)
        self._destroy()

    @ZxdgToplevelDecorationV1.request(
        Argument(ArgumentType.Uint),
    )
    def set_mode(self, mode: int) -> None:
        """Set the decoration mode

        Set the toplevel surface decoration mode. This informs the compositor
        that the client prefers the provided decoration mode.

        After requesting a decoration mode, the compositor will respond by
        emitting an :func:`XdgSurface.configure()
        <pywayland.protocol.xdg_shell.XdgSurface.configure>` event. The client
        should then update its content, drawing it without decorations if the
        received mode is server-side decorations. The client must also
        acknowledge the configure when committing the new content (see
        :func:`XdgSurface.ack_configure()
        <pywayland.protocol.xdg_shell.XdgSurface.ack_configure>`).

        The compositor can decide not to use the client's mode and enforce a
        different mode instead.

        Clients whose decoration mode depend on the
        :class:`~pywayland.protocol.xdg_shell.XdgToplevel` state may send a
        set_mode request in response to an :func:`XdgSurface.configure()
        <pywayland.protocol.xdg_shell.XdgSurface.configure>` event and wait for
        the next :func:`XdgSurface.configure()
        <pywayland.protocol.xdg_shell.XdgSurface.configure>` event to prevent
        unwanted state. Such clients are responsible for preventing configure
        loops and must make sure not to send multiple successive set_mode
        requests with the same decoration mode.

        If an invalid mode is supplied by the client, the invalid_mode protocol
        error is raised by the compositor.

        :param mode:
            the decoration mode
        :type mode:
            `ArgumentType.Uint`
        """
        self._marshal(1, mode)

    @ZxdgToplevelDecorationV1.request()
    def unset_mode(self) -> None:
        """Unset the decoration mode

        Unset the toplevel surface decoration mode. This informs the compositor
        that the client doesn't prefer a particular decoration mode.

        This request has the same semantics as set_mode.
        """
        self._marshal(2)


class ZxdgDecorationManagerV1Global(Global[ZxdgDecorationManagerV1]):
    interface = ZxdgDecorationManagerV1


class ZxdgToplevelDecorationV1Global(Global[ZxdgToplevelDecorationV1]):
    interface = ZxdgToplevelDecorationV1


ZxdgDecorationManagerV1._gen_c()
ZxdgDecorationManagerV1.proxy_class = ZxdgDecorationManagerV1Proxy
ZxdgDecorationManagerV1.resource_class = ZxdgDecorationManagerV1Resource
ZxdgDecorationManagerV1.global_class = ZxdgDecorationManagerV1Global


ZxdgToplevelDecorationV1._gen_c()
ZxdgToplevelDecorationV1.proxy_class = ZxdgToplevelDecorationV1Proxy
ZxdgToplevelDecorationV1.resource_class = ZxdgToplevelDecorationV1Resource
ZxdgToplevelDecorationV1.global_class = ZxdgToplevelDecorationV1Global
